'''
Neuron simulator export for:

Components:
    null (Type: notes)
    ifcell (Type: IF_curr_alpha:  tau_refrac=2.0 (dimensionless) v_thresh=20.0 (dimensionless) tau_m=20.0 (dimensionless) v_rest=0.0 (dimensionless) v_reset=0.0 (dimensionless) cm=0.001 (dimensionless) i_offset=0.0 (dimensionless) tau_syn_E=0.1 (dimensionless) tau_syn_I=0.1 (dimensionless) v_init=0.0 (dimensionless) MSEC=0.001 (SI time) MVOLT=0.001 (SI voltage) NFARAD=1.0E-9 (SI capacitance))
    ampa (Type: alphaCurrSynapse:  tau_syn=0.1 (dimensionless) MSEC=0.001 (SI time) MVOLT=0.001 (SI voltage) NAMP=1.0E-9 (SI current))
    gaba (Type: alphaCurrSynapse:  tau_syn=0.1 (dimensionless) MSEC=0.001 (SI time) MVOLT=0.001 (SI voltage) NAMP=1.0E-9 (SI current))
    poisson_input (Type: SpikeSourcePoisson:  start=0.0 (SI time) duration=1000000.0 (SI time) rate=10000.0 (SI per_time) end=1000000.0 (SI time) LONG_TIME=3.6E12 (SI time) SMALL_TIME=1.0000000000000002E-12 (SI time))
    Example7_Brunel2000 (Type: network)
    SimExample7 (Type: Simulation:  length=1.0 (SI time) step=2.5E-5 (SI time))


    This NEURON file has been generated by org.neuroml.export (see https://github.com/NeuroML/org.neuroml.export)
         org.neuroml.export  v1.8.1
         org.neuroml.model   v1.8.1
         jLEMS               v0.10.6

'''

import neuron

import time
import datetime
import sys

import hashlib
h = neuron.h
h.load_file("stdlib.hoc")

h.load_file("stdgui.hoc")

h("objref p")
h("p = new PythonObject()")

class NeuronSimulation():

    def __init__(self, tstop, dt, seed=123):

        print("\n    Starting simulation in NEURON of %sms generated from NeuroML2 model...\n"%tstop)

        self.setup_start = time.time()
        self.seed = seed
        import socket
        self.report_file = open('report.SimExample7.txt','w')
        print('Simulator version:  %s'%h.nrnversion())
        self.report_file.write('# Report of running simulation with %s\n'%h.nrnversion())
        self.report_file.write('Simulator=NEURON\n')
        self.report_file.write('SimulatorVersion=%s\n'%h.nrnversion())

        self.report_file.write('SimulationFile=%s\n'%__file__)
        self.report_file.write('PythonVersion=%s\n'%sys.version.replace('\n',' '))
        print('Python version:     %s'%sys.version.replace('\n',' '))
        self.report_file.write('NeuroMLExportVersion=1.8.1\n')
        self.report_file.write('SimulationSeed=%s\n'%self.seed)
        self.report_file.write('Hostname=%s\n'%socket.gethostname())
        self.randoms = []
        self.next_global_id = 0  # Used in Random123 classes for elements using random(), etc. 

        self.next_spiking_input_id = 0  # Used in Random123 classes for elements using random(), etc. 

        '''
        Adding simulation Component(id=SimExample7 type=Simulation) of network/component: Example7_Brunel2000 (Type: network)
        
        '''
        # ######################   Population: Epop
        print("Population Epop contains 20 instance(s) of component: ifcell of type: IF_curr_alpha")

        h(" {n_Epop = 20} ")
        '''
        Population Epop contains instances of Component(id=ifcell type=IF_curr_alpha)
        whose dynamics will be implemented as a mechanism (ifcell) in a mod file
        '''
        h(" create Epop[20]")
        h(" objectvar m_ifcell_Epop[20] ")

        for i in range(int(h.n_Epop)):
            h.Epop[i].L = 10.0
            h.Epop[i](0.5).diam = 10.0
            h.Epop[i](0.5).cm = 0.3183098861837907
            h.Epop[i].push()
            h(" Epop[%i]  { m_ifcell_Epop[%i] = new ifcell(0.5) } "%(i,i))

            h.m_ifcell_Epop[i].tau_refrac = 2.0
            h.m_ifcell_Epop[i].v_thresh = 20.0
            h.m_ifcell_Epop[i].tau_m = 20.0
            h.m_ifcell_Epop[i].v_rest = 0.0
            h.m_ifcell_Epop[i].v_reset = 0.0
            h.m_ifcell_Epop[i].cm = 0.001
            h.m_ifcell_Epop[i].i_offset = 0.0
            h.m_ifcell_Epop[i].tau_syn_E = 0.1
            h.m_ifcell_Epop[i].tau_syn_I = 0.1
            h.m_ifcell_Epop[i].v_init = 0.0
            h.m_ifcell_Epop[i].MSEC = 1.0
            h.m_ifcell_Epop[i].MVOLT = 1.0
            h.m_ifcell_Epop[i].NFARAD = 0.001
            h.pop_section()

            self.next_global_id+=1

        h(" Epop[0] { pt3dclear() } ")
        h(" Epop[0] { pt3dadd(966.45355, 44.07326 + (5), 7.49147, 10) } ")
        h(" Epop[0] { pt3dadd(966.45355, 44.07326 + (-5), 7.49147, 10) } ")
        h(" Epop[1] { pt3dclear() } ")
        h(" Epop[1] { pt3dadd(910.97595, 93.9269 + (5), 582.2276, 10) } ")
        h(" Epop[1] { pt3dadd(910.97595, 93.9269 + (-5), 582.2276, 10) } ")
        h(" Epop[2] { pt3dclear() } ")
        h(" Epop[2] { pt3dadd(671.5635, 8.393823 + (5), 766.48096, 10) } ")
        h(" Epop[2] { pt3dadd(671.5635, 8.393823 + (-5), 766.48096, 10) } ")
        h(" Epop[3] { pt3dclear() } ")
        h(" Epop[3] { pt3dadd(236.80977, 3.081402 + (5), 788.7727, 10) } ")
        h(" Epop[3] { pt3dadd(236.80977, 3.081402 + (-5), 788.7727, 10) } ")
        h(" Epop[4] { pt3dclear() } ")
        h(" Epop[4] { pt3dadd(346.08896, 62.328148 + (5), 615.8157, 10) } ")
        h(" Epop[4] { pt3dadd(346.08896, 62.328148 + (-5), 615.8157, 10) } ")
        h(" Epop[5] { pt3dclear() } ")
        h(" Epop[5] { pt3dadd(148.55464, 18.309065 + (5), 114.41297, 10) } ")
        h(" Epop[5] { pt3dadd(148.55464, 18.309065 + (-5), 114.41297, 10) } ")
        h(" Epop[6] { pt3dclear() } ")
        h(" Epop[6] { pt3dadd(14.61878, 48.675156 + (5), 964.90155, 10) } ")
        h(" Epop[6] { pt3dadd(14.61878, 48.675156 + (-5), 964.90155, 10) } ")
        h(" Epop[7] { pt3dclear() } ")
        h(" Epop[7] { pt3dadd(64.56228, 54.108818 + (5), 465.89856, 10) } ")
        h(" Epop[7] { pt3dadd(64.56228, 54.108818 + (-5), 465.89856, 10) } ")
        h(" Epop[8] { pt3dclear() } ")
        h(" Epop[8] { pt3dadd(601.46344, 8.892883 + (5), 579.0027, 10) } ")
        h(" Epop[8] { pt3dadd(601.46344, 8.892883 + (-5), 579.0027, 10) } ")
        h(" Epop[9] { pt3dclear() } ")
        h(" Epop[9] { pt3dadd(269.5855, 55.643257 + (5), 644.6342, 10) } ")
        h(" Epop[9] { pt3dadd(269.5855, 55.643257 + (-5), 644.6342, 10) } ")
        h(" Epop[10] { pt3dclear() } ")
        h(" Epop[10] { pt3dadd(481.03638, 35.523914 + (5), 249.15211, 10) } ")
        h(" Epop[10] { pt3dadd(481.03638, 35.523914 + (-5), 249.15211, 10) } ")
        h(" Epop[11] { pt3dclear() } ")
        h(" Epop[11] { pt3dadd(933.5155, 45.338802 + (5), 530.1612, 10) } ")
        h(" Epop[11] { pt3dadd(933.5155, 45.338802 + (-5), 530.1612, 10) } ")
        h(" Epop[12] { pt3dclear() } ")
        h(" Epop[12] { pt3dadd(19.299566, 50.810192 + (5), 5.780237, 10) } ")
        h(" Epop[12] { pt3dadd(19.299566, 50.810192 + (-5), 5.780237, 10) } ")
        h(" Epop[13] { pt3dclear() } ")
        h(" Epop[13] { pt3dadd(143.76843, 47.282692 + (5), 377.34744, 10) } ")
        h(" Epop[13] { pt3dadd(143.76843, 47.282692 + (-5), 377.34744, 10) } ")
        h(" Epop[14] { pt3dclear() } ")
        h(" Epop[14] { pt3dadd(54.175198, 58.75285 + (5), 164.00322, 10) } ")
        h(" Epop[14] { pt3dadd(54.175198, 58.75285 + (-5), 164.00322, 10) } ")
        h(" Epop[15] { pt3dclear() } ")
        h(" Epop[15] { pt3dadd(557.33026, 14.424572 + (5), 937.30707, 10) } ")
        h(" Epop[15] { pt3dadd(557.33026, 14.424572 + (-5), 937.30707, 10) } ")
        h(" Epop[16] { pt3dclear() } ")
        h(" Epop[16] { pt3dadd(770.98, 95.69331 + (5), 141.22777, 10) } ")
        h(" Epop[16] { pt3dadd(770.98, 95.69331 + (-5), 141.22777, 10) } ")
        h(" Epop[17] { pt3dclear() } ")
        h(" Epop[17] { pt3dadd(305.3927, 3.9589624 + (5), 276.7837, 10) } ")
        h(" Epop[17] { pt3dadd(305.3927, 3.9589624 + (-5), 276.7837, 10) } ")
        h(" Epop[18] { pt3dclear() } ")
        h(" Epop[18] { pt3dadd(806.5125, 17.734304 + (5), 154.57051, 10) } ")
        h(" Epop[18] { pt3dadd(806.5125, 17.734304 + (-5), 154.57051, 10) } ")
        h(" Epop[19] { pt3dclear() } ")
        h(" Epop[19] { pt3dadd(954.7186, 15.45514 + (5), 833.8893, 10) } ")
        h(" Epop[19] { pt3dadd(954.7186, 15.45514 + (-5), 833.8893, 10) } ")

        # ######################   Population: expoisson
        print("Population expoisson contains 20 instance(s) of component: poisson_input of type: SpikeSourcePoisson")

        h(" {n_expoisson = 20} ")
        '''
        Population expoisson contains instances of Component(id=poisson_input type=SpikeSourcePoisson)
        whose dynamics will be implemented as a mechanism (poisson_input) in a mod file
        '''
        h(" create expoisson[20]")
        h(" objectvar m_poisson_input_expoisson[20] ")

        for i in range(int(h.n_expoisson)):
            h.expoisson[i].L = 10.0
            h.expoisson[i](0.5).diam = 10.0
            h.expoisson[i](0.5).cm = 318.31
            h.expoisson[i].push()
            h(" expoisson[%i]  { m_poisson_input_expoisson[%i] = new poisson_input(0.5) } "%(i,i))

            h.m_poisson_input_expoisson[i].start = 0.0
            h.m_poisson_input_expoisson[i].duration = 1.0E9
            h.m_poisson_input_expoisson[i].rate = 10.0
            h.m_poisson_input_expoisson[i].end = 1.0E9
            h.m_poisson_input_expoisson[i].LONG_TIME = 3.59999998E15
            h.m_poisson_input_expoisson[i].SMALL_TIME = 1.0E-9
            # Spiking element (SpikeSourcePoisson), will require seeding...
            rand = h.Random()
            self.randoms.append(rand)
            #print("Seeding random generator on m_poisson_input_expoisson[i] (i=%i) with stim seed %s"%(i, self.seed))
            self._init_stim_randomizer(rand,"expoisson",i, self.seed)
            rand.negexp(1)
            h.m_poisson_input_expoisson[i].noiseFromRandom(rand)

            h.pop_section()

            self.next_global_id+=1

        h(" expoisson[0] { pt3dclear() } ")
        h(" expoisson[0] { pt3dadd(41.062805, 38.61835 + (5), 349.59232, 10) } ")
        h(" expoisson[0] { pt3dadd(41.062805, 38.61835 + (-5), 349.59232, 10) } ")
        h(" expoisson[1] { pt3dclear() } ")
        h(" expoisson[1] { pt3dadd(341.7078, 81.64631 + (5), 475.9349, 10) } ")
        h(" expoisson[1] { pt3dadd(341.7078, 81.64631 + (-5), 475.9349, 10) } ")
        h(" expoisson[2] { pt3dclear() } ")
        h(" expoisson[2] { pt3dadd(782.8903, 47.08401 + (5), 817.344, 10) } ")
        h(" expoisson[2] { pt3dadd(782.8903, 47.08401 + (-5), 817.344, 10) } ")
        h(" expoisson[3] { pt3dclear() } ")
        h(" expoisson[3] { pt3dadd(881.5676, 43.95964 + (5), 781.0636, 10) } ")
        h(" expoisson[3] { pt3dadd(881.5676, 43.95964 + (-5), 781.0636, 10) } ")
        h(" expoisson[4] { pt3dclear() } ")
        h(" expoisson[4] { pt3dadd(814.74115, 29.567892 + (5), 123.87667, 10) } ")
        h(" expoisson[4] { pt3dadd(814.74115, 29.567892 + (-5), 123.87667, 10) } ")
        h(" expoisson[5] { pt3dclear() } ")
        h(" expoisson[5] { pt3dadd(185.62186, 43.60598 + (5), 119.46741, 10) } ")
        h(" expoisson[5] { pt3dadd(185.62186, 43.60598 + (-5), 119.46741, 10) } ")
        h(" expoisson[6] { pt3dclear() } ")
        h(" expoisson[6] { pt3dadd(529.79913, 82.942116 + (5), 485.20114, 10) } ")
        h(" expoisson[6] { pt3dadd(529.79913, 82.942116 + (-5), 485.20114, 10) } ")
        h(" expoisson[7] { pt3dclear() } ")
        h(" expoisson[7] { pt3dadd(817.734, 65.63916 + (5), 641.043, 10) } ")
        h(" expoisson[7] { pt3dadd(817.734, 65.63916 + (-5), 641.043, 10) } ")
        h(" expoisson[8] { pt3dclear() } ")
        h(" expoisson[8] { pt3dadd(345.34894, 70.26589 + (5), 809.9406, 10) } ")
        h(" expoisson[8] { pt3dadd(345.34894, 70.26589 + (-5), 809.9406, 10) } ")
        h(" expoisson[9] { pt3dclear() } ")
        h(" expoisson[9] { pt3dadd(157.16959, 90.799065 + (5), 269.33035, 10) } ")
        h(" expoisson[9] { pt3dadd(157.16959, 90.799065 + (-5), 269.33035, 10) } ")
        h(" expoisson[10] { pt3dclear() } ")
        h(" expoisson[10] { pt3dadd(154.8465, 84.04792 + (5), 720.128, 10) } ")
        h(" expoisson[10] { pt3dadd(154.8465, 84.04792 + (-5), 720.128, 10) } ")
        h(" expoisson[11] { pt3dclear() } ")
        h(" expoisson[11] { pt3dadd(793.5624, 44.667065 + (5), 70.77389, 10) } ")
        h(" expoisson[11] { pt3dadd(793.5624, 44.667065 + (-5), 70.77389, 10) } ")
        h(" expoisson[12] { pt3dclear() } ")
        h(" expoisson[12] { pt3dadd(395.24304, 4.7735295 + (5), 286.1335, 10) } ")
        h(" expoisson[12] { pt3dadd(395.24304, 4.7735295 + (-5), 286.1335, 10) } ")
        h(" expoisson[13] { pt3dclear() } ")
        h(" expoisson[13] { pt3dadd(37.994595, 50.671486 + (5), 87.6678, 10) } ")
        h(" expoisson[13] { pt3dadd(37.994595, 50.671486 + (-5), 87.6678, 10) } ")
        h(" expoisson[14] { pt3dclear() } ")
        h(" expoisson[14] { pt3dadd(932.8256, 69.93873 + (5), 316.98602, 10) } ")
        h(" expoisson[14] { pt3dadd(932.8256, 69.93873 + (-5), 316.98602, 10) } ")
        h(" expoisson[15] { pt3dclear() } ")
        h(" expoisson[15] { pt3dadd(945.0511, 6.6227603 + (5), 257.4289, 10) } ")
        h(" expoisson[15] { pt3dadd(945.0511, 6.6227603 + (-5), 257.4289, 10) } ")
        h(" expoisson[16] { pt3dclear() } ")
        h(" expoisson[16] { pt3dadd(72.99965, 42.630424 + (5), 201.89677, 10) } ")
        h(" expoisson[16] { pt3dadd(72.99965, 42.630424 + (-5), 201.89677, 10) } ")
        h(" expoisson[17] { pt3dclear() } ")
        h(" expoisson[17] { pt3dadd(396.64664, 70.48599 + (5), 887.35333, 10) } ")
        h(" expoisson[17] { pt3dadd(396.64664, 70.48599 + (-5), 887.35333, 10) } ")
        h(" expoisson[18] { pt3dclear() } ")
        h(" expoisson[18] { pt3dadd(500.5178, 81.85316 + (5), 361.25452, 10) } ")
        h(" expoisson[18] { pt3dadd(500.5178, 81.85316 + (-5), 361.25452, 10) } ")
        h(" expoisson[19] { pt3dclear() } ")
        h(" expoisson[19] { pt3dadd(858.68616, 51.503418 + (5), 702.41547, 10) } ")
        h(" expoisson[19] { pt3dadd(858.68616, 51.503418 + (-5), 702.41547, 10) } ")

        # ######################   Population: Ipop
        print("Population Ipop contains 5 instance(s) of component: ifcell of type: IF_curr_alpha")

        h(" {n_Ipop = 5} ")
        '''
        Population Ipop contains instances of Component(id=ifcell type=IF_curr_alpha)
        whose dynamics will be implemented as a mechanism (ifcell) in a mod file
        '''
        h(" create Ipop[5]")
        h(" objectvar m_ifcell_Ipop[5] ")

        for i in range(int(h.n_Ipop)):
            h.Ipop[i].L = 10.0
            h.Ipop[i](0.5).diam = 10.0
            h.Ipop[i](0.5).cm = 0.3183098861837907
            h.Ipop[i].push()
            h(" Ipop[%i]  { m_ifcell_Ipop[%i] = new ifcell(0.5) } "%(i,i))

            h.m_ifcell_Ipop[i].tau_refrac = 2.0
            h.m_ifcell_Ipop[i].v_thresh = 20.0
            h.m_ifcell_Ipop[i].tau_m = 20.0
            h.m_ifcell_Ipop[i].v_rest = 0.0
            h.m_ifcell_Ipop[i].v_reset = 0.0
            h.m_ifcell_Ipop[i].cm = 0.001
            h.m_ifcell_Ipop[i].i_offset = 0.0
            h.m_ifcell_Ipop[i].tau_syn_E = 0.1
            h.m_ifcell_Ipop[i].tau_syn_I = 0.1
            h.m_ifcell_Ipop[i].v_init = 0.0
            h.m_ifcell_Ipop[i].MSEC = 1.0
            h.m_ifcell_Ipop[i].MVOLT = 1.0
            h.m_ifcell_Ipop[i].NFARAD = 0.001
            h.pop_section()

            self.next_global_id+=1

        h(" Ipop[0] { pt3dclear() } ")
        h(" Ipop[0] { pt3dadd(174.68991, 58.45118 + (5), 301.77707, 10) } ")
        h(" Ipop[0] { pt3dadd(174.68991, 58.45118 + (-5), 301.77707, 10) } ")
        h(" Ipop[1] { pt3dclear() } ")
        h(" Ipop[1] { pt3dadd(812.8514, 53.434074 + (5), 499.5873, 10) } ")
        h(" Ipop[1] { pt3dadd(812.8514, 53.434074 + (-5), 499.5873, 10) } ")
        h(" Ipop[2] { pt3dclear() } ")
        h(" Ipop[2] { pt3dadd(773.3252, 54.90307 + (5), 333.75803, 10) } ")
        h(" Ipop[2] { pt3dadd(773.3252, 54.90307 + (-5), 333.75803, 10) } ")
        h(" Ipop[3] { pt3dclear() } ")
        h(" Ipop[3] { pt3dadd(130.93286, 62.4506 + (5), 925.3462, 10) } ")
        h(" Ipop[3] { pt3dadd(130.93286, 62.4506 + (-5), 925.3462, 10) } ")
        h(" Ipop[4] { pt3dclear() } ")
        h(" Ipop[4] { pt3dadd(842.33887, 6.9749784 + (5), 325.28522, 10) } ")
        h(" Ipop[4] { pt3dadd(842.33887, 6.9749784 + (-5), 325.28522, 10) } ")

        # ######################   Population: inpoisson
        print("Population inpoisson contains 5 instance(s) of component: poisson_input of type: SpikeSourcePoisson")

        h(" {n_inpoisson = 5} ")
        '''
        Population inpoisson contains instances of Component(id=poisson_input type=SpikeSourcePoisson)
        whose dynamics will be implemented as a mechanism (poisson_input) in a mod file
        '''
        h(" create inpoisson[5]")
        h(" objectvar m_poisson_input_inpoisson[5] ")

        for i in range(int(h.n_inpoisson)):
            h.inpoisson[i].L = 10.0
            h.inpoisson[i](0.5).diam = 10.0
            h.inpoisson[i](0.5).cm = 318.31
            h.inpoisson[i].push()
            h(" inpoisson[%i]  { m_poisson_input_inpoisson[%i] = new poisson_input(0.5) } "%(i,i))

            h.m_poisson_input_inpoisson[i].start = 0.0
            h.m_poisson_input_inpoisson[i].duration = 1.0E9
            h.m_poisson_input_inpoisson[i].rate = 10.0
            h.m_poisson_input_inpoisson[i].end = 1.0E9
            h.m_poisson_input_inpoisson[i].LONG_TIME = 3.59999998E15
            h.m_poisson_input_inpoisson[i].SMALL_TIME = 1.0E-9
            # Spiking element (SpikeSourcePoisson), will require seeding...
            rand = h.Random()
            self.randoms.append(rand)
            #print("Seeding random generator on m_poisson_input_inpoisson[i] (i=%i) with stim seed %s"%(i, self.seed))
            self._init_stim_randomizer(rand,"inpoisson",i, self.seed)
            rand.negexp(1)
            h.m_poisson_input_inpoisson[i].noiseFromRandom(rand)

            h.pop_section()

            self.next_global_id+=1

        h(" inpoisson[0] { pt3dclear() } ")
        h(" inpoisson[0] { pt3dadd(1.7063742, 67.46972 + (5), 638.2801, 10) } ")
        h(" inpoisson[0] { pt3dadd(1.7063742, 67.46972 + (-5), 638.2801, 10) } ")
        h(" inpoisson[1] { pt3dclear() } ")
        h(" inpoisson[1] { pt3dadd(757.90875, 14.827393 + (5), 217.03369, 10) } ")
        h(" inpoisson[1] { pt3dadd(757.90875, 14.827393 + (-5), 217.03369, 10) } ")
        h(" inpoisson[2] { pt3dclear() } ")
        h(" inpoisson[2] { pt3dadd(432.77237, 73.672485 + (5), 207.119, 10) } ")
        h(" inpoisson[2] { pt3dadd(432.77237, 73.672485 + (-5), 207.119, 10) } ")
        h(" inpoisson[3] { pt3dclear() } ")
        h(" inpoisson[3] { pt3dadd(822.8369, 38.12011 + (5), 874.28577, 10) } ")
        h(" inpoisson[3] { pt3dadd(822.8369, 38.12011 + (-5), 874.28577, 10) } ")
        h(" inpoisson[4] { pt3dclear() } ")
        h(" inpoisson[4] { pt3dadd(960.02466, 53.74877 + (5), 921.0674, 10) } ")
        h(" inpoisson[4] { pt3dadd(960.02466, 53.74877 + (-5), 921.0674, 10) } ")

        # ######################   Projection: projEinput
        print("Adding projection: projEinput, from expoisson to Epop with synapse ampa, 20 connection(s)")

        h("objectvar syn_projEinput_ampa[20]")

        h("objectvar netConn_projEinput_ampa[20]")

        # Connection 0: cell 0, seg 0 (0.5) [0.5 on expoisson[0]] -> cell 0, seg 0 (0.5) [0.5 on Epop[0]], weight: 4.1363508E-4, delay 0.025
        h("Epop[0] syn_projEinput_ampa[0] = new ampa(0.5)")
        h("expoisson[0] netConn_projEinput_ampa[0] = new NetCon(m_poisson_input_expoisson[0], syn_projEinput_ampa[0], 0.0, 0.025, 4.1363508E-4)")  

        # Connection 1: cell 1, seg 0 (0.5) [0.5 on expoisson[1]] -> cell 1, seg 0 (0.5) [0.5 on Epop[1]], weight: 4.1363508E-4, delay 0.025
        h("Epop[1] syn_projEinput_ampa[1] = new ampa(0.5)")
        h("expoisson[1] netConn_projEinput_ampa[1] = new NetCon(m_poisson_input_expoisson[1], syn_projEinput_ampa[1], 0.0, 0.025, 4.1363508E-4)")  

        # Connection 2: cell 2, seg 0 (0.5) [0.5 on expoisson[2]] -> cell 2, seg 0 (0.5) [0.5 on Epop[2]], weight: 4.1363508E-4, delay 0.025
        h("Epop[2] syn_projEinput_ampa[2] = new ampa(0.5)")
        h("expoisson[2] netConn_projEinput_ampa[2] = new NetCon(m_poisson_input_expoisson[2], syn_projEinput_ampa[2], 0.0, 0.025, 4.1363508E-4)")  

        # Connection 3: cell 3, seg 0 (0.5) [0.5 on expoisson[3]] -> cell 3, seg 0 (0.5) [0.5 on Epop[3]], weight: 4.1363508E-4, delay 0.025
        h("Epop[3] syn_projEinput_ampa[3] = new ampa(0.5)")
        h("expoisson[3] netConn_projEinput_ampa[3] = new NetCon(m_poisson_input_expoisson[3], syn_projEinput_ampa[3], 0.0, 0.025, 4.1363508E-4)")  

        # Connection 4: cell 4, seg 0 (0.5) [0.5 on expoisson[4]] -> cell 4, seg 0 (0.5) [0.5 on Epop[4]], weight: 4.1363508E-4, delay 0.025
        h("Epop[4] syn_projEinput_ampa[4] = new ampa(0.5)")
        h("expoisson[4] netConn_projEinput_ampa[4] = new NetCon(m_poisson_input_expoisson[4], syn_projEinput_ampa[4], 0.0, 0.025, 4.1363508E-4)")  

        # Connection 5: cell 5, seg 0 (0.5) [0.5 on expoisson[5]] -> cell 5, seg 0 (0.5) [0.5 on Epop[5]], weight: 4.1363508E-4, delay 0.025
        h("Epop[5] syn_projEinput_ampa[5] = new ampa(0.5)")
        h("expoisson[5] netConn_projEinput_ampa[5] = new NetCon(m_poisson_input_expoisson[5], syn_projEinput_ampa[5], 0.0, 0.025, 4.1363508E-4)")  

        # Connection 6: cell 6, seg 0 (0.5) [0.5 on expoisson[6]] -> cell 6, seg 0 (0.5) [0.5 on Epop[6]], weight: 4.1363508E-4, delay 0.025
        h("Epop[6] syn_projEinput_ampa[6] = new ampa(0.5)")
        h("expoisson[6] netConn_projEinput_ampa[6] = new NetCon(m_poisson_input_expoisson[6], syn_projEinput_ampa[6], 0.0, 0.025, 4.1363508E-4)")  

        # Connection 7: cell 7, seg 0 (0.5) [0.5 on expoisson[7]] -> cell 7, seg 0 (0.5) [0.5 on Epop[7]], weight: 4.1363508E-4, delay 0.025
        h("Epop[7] syn_projEinput_ampa[7] = new ampa(0.5)")
        h("expoisson[7] netConn_projEinput_ampa[7] = new NetCon(m_poisson_input_expoisson[7], syn_projEinput_ampa[7], 0.0, 0.025, 4.1363508E-4)")  

        # Connection 8: cell 8, seg 0 (0.5) [0.5 on expoisson[8]] -> cell 8, seg 0 (0.5) [0.5 on Epop[8]], weight: 4.1363508E-4, delay 0.025
        h("Epop[8] syn_projEinput_ampa[8] = new ampa(0.5)")
        h("expoisson[8] netConn_projEinput_ampa[8] = new NetCon(m_poisson_input_expoisson[8], syn_projEinput_ampa[8], 0.0, 0.025, 4.1363508E-4)")  

        # Connection 9: cell 9, seg 0 (0.5) [0.5 on expoisson[9]] -> cell 9, seg 0 (0.5) [0.5 on Epop[9]], weight: 4.1363508E-4, delay 0.025
        h("Epop[9] syn_projEinput_ampa[9] = new ampa(0.5)")
        h("expoisson[9] netConn_projEinput_ampa[9] = new NetCon(m_poisson_input_expoisson[9], syn_projEinput_ampa[9], 0.0, 0.025, 4.1363508E-4)")  

        # Connection 10: cell 10, seg 0 (0.5) [0.5 on expoisson[10]] -> cell 10, seg 0 (0.5) [0.5 on Epop[10]], weight: 4.1363508E-4, delay 0.025
        h("Epop[10] syn_projEinput_ampa[10] = new ampa(0.5)")
        h("expoisson[10] netConn_projEinput_ampa[10] = new NetCon(m_poisson_input_expoisson[10], syn_projEinput_ampa[10], 0.0, 0.025, 4.1363508E-4)")  

        # Connection 11: cell 11, seg 0 (0.5) [0.5 on expoisson[11]] -> cell 11, seg 0 (0.5) [0.5 on Epop[11]], weight: 4.1363508E-4, delay 0.025
        h("Epop[11] syn_projEinput_ampa[11] = new ampa(0.5)")
        h("expoisson[11] netConn_projEinput_ampa[11] = new NetCon(m_poisson_input_expoisson[11], syn_projEinput_ampa[11], 0.0, 0.025, 4.1363508E-4)")  

        # Connection 12: cell 12, seg 0 (0.5) [0.5 on expoisson[12]] -> cell 12, seg 0 (0.5) [0.5 on Epop[12]], weight: 4.1363508E-4, delay 0.025
        h("Epop[12] syn_projEinput_ampa[12] = new ampa(0.5)")
        h("expoisson[12] netConn_projEinput_ampa[12] = new NetCon(m_poisson_input_expoisson[12], syn_projEinput_ampa[12], 0.0, 0.025, 4.1363508E-4)")  

        # Connection 13: cell 13, seg 0 (0.5) [0.5 on expoisson[13]] -> cell 13, seg 0 (0.5) [0.5 on Epop[13]], weight: 4.1363508E-4, delay 0.025
        h("Epop[13] syn_projEinput_ampa[13] = new ampa(0.5)")
        h("expoisson[13] netConn_projEinput_ampa[13] = new NetCon(m_poisson_input_expoisson[13], syn_projEinput_ampa[13], 0.0, 0.025, 4.1363508E-4)")  

        # Connection 14: cell 14, seg 0 (0.5) [0.5 on expoisson[14]] -> cell 14, seg 0 (0.5) [0.5 on Epop[14]], weight: 4.1363508E-4, delay 0.025
        h("Epop[14] syn_projEinput_ampa[14] = new ampa(0.5)")
        h("expoisson[14] netConn_projEinput_ampa[14] = new NetCon(m_poisson_input_expoisson[14], syn_projEinput_ampa[14], 0.0, 0.025, 4.1363508E-4)")  

        # Connection 15: cell 15, seg 0 (0.5) [0.5 on expoisson[15]] -> cell 15, seg 0 (0.5) [0.5 on Epop[15]], weight: 4.1363508E-4, delay 0.025
        h("Epop[15] syn_projEinput_ampa[15] = new ampa(0.5)")
        h("expoisson[15] netConn_projEinput_ampa[15] = new NetCon(m_poisson_input_expoisson[15], syn_projEinput_ampa[15], 0.0, 0.025, 4.1363508E-4)")  

        # Connection 16: cell 16, seg 0 (0.5) [0.5 on expoisson[16]] -> cell 16, seg 0 (0.5) [0.5 on Epop[16]], weight: 4.1363508E-4, delay 0.025
        h("Epop[16] syn_projEinput_ampa[16] = new ampa(0.5)")
        h("expoisson[16] netConn_projEinput_ampa[16] = new NetCon(m_poisson_input_expoisson[16], syn_projEinput_ampa[16], 0.0, 0.025, 4.1363508E-4)")  

        # Connection 17: cell 17, seg 0 (0.5) [0.5 on expoisson[17]] -> cell 17, seg 0 (0.5) [0.5 on Epop[17]], weight: 4.1363508E-4, delay 0.025
        h("Epop[17] syn_projEinput_ampa[17] = new ampa(0.5)")
        h("expoisson[17] netConn_projEinput_ampa[17] = new NetCon(m_poisson_input_expoisson[17], syn_projEinput_ampa[17], 0.0, 0.025, 4.1363508E-4)")  

        # Connection 18: cell 18, seg 0 (0.5) [0.5 on expoisson[18]] -> cell 18, seg 0 (0.5) [0.5 on Epop[18]], weight: 4.1363508E-4, delay 0.025
        h("Epop[18] syn_projEinput_ampa[18] = new ampa(0.5)")
        h("expoisson[18] netConn_projEinput_ampa[18] = new NetCon(m_poisson_input_expoisson[18], syn_projEinput_ampa[18], 0.0, 0.025, 4.1363508E-4)")  

        # Connection 19: cell 19, seg 0 (0.5) [0.5 on expoisson[19]] -> cell 19, seg 0 (0.5) [0.5 on Epop[19]], weight: 4.1363508E-4, delay 0.025
        h("Epop[19] syn_projEinput_ampa[19] = new ampa(0.5)")
        h("expoisson[19] netConn_projEinput_ampa[19] = new NetCon(m_poisson_input_expoisson[19], syn_projEinput_ampa[19], 0.0, 0.025, 4.1363508E-4)")  

        # ######################   Projection: projIinput
        print("Adding projection: projIinput, from inpoisson to Ipop with synapse ampa, 5 connection(s)")

        h("objectvar syn_projIinput_ampa[5]")

        h("objectvar netConn_projIinput_ampa[5]")

        # Connection 0: cell 0, seg 0 (0.5) [0.5 on inpoisson[0]] -> cell 0, seg 0 (0.5) [0.5 on Ipop[0]], weight: 4.1363508E-4, delay 0.025
        h("Ipop[0] syn_projIinput_ampa[0] = new ampa(0.5)")
        h("inpoisson[0] netConn_projIinput_ampa[0] = new NetCon(m_poisson_input_inpoisson[0], syn_projIinput_ampa[0], 0.0, 0.025, 4.1363508E-4)")  

        # Connection 1: cell 1, seg 0 (0.5) [0.5 on inpoisson[1]] -> cell 1, seg 0 (0.5) [0.5 on Ipop[1]], weight: 4.1363508E-4, delay 0.025
        h("Ipop[1] syn_projIinput_ampa[1] = new ampa(0.5)")
        h("inpoisson[1] netConn_projIinput_ampa[1] = new NetCon(m_poisson_input_inpoisson[1], syn_projIinput_ampa[1], 0.0, 0.025, 4.1363508E-4)")  

        # Connection 2: cell 2, seg 0 (0.5) [0.5 on inpoisson[2]] -> cell 2, seg 0 (0.5) [0.5 on Ipop[2]], weight: 4.1363508E-4, delay 0.025
        h("Ipop[2] syn_projIinput_ampa[2] = new ampa(0.5)")
        h("inpoisson[2] netConn_projIinput_ampa[2] = new NetCon(m_poisson_input_inpoisson[2], syn_projIinput_ampa[2], 0.0, 0.025, 4.1363508E-4)")  

        # Connection 3: cell 3, seg 0 (0.5) [0.5 on inpoisson[3]] -> cell 3, seg 0 (0.5) [0.5 on Ipop[3]], weight: 4.1363508E-4, delay 0.025
        h("Ipop[3] syn_projIinput_ampa[3] = new ampa(0.5)")
        h("inpoisson[3] netConn_projIinput_ampa[3] = new NetCon(m_poisson_input_inpoisson[3], syn_projIinput_ampa[3], 0.0, 0.025, 4.1363508E-4)")  

        # Connection 4: cell 4, seg 0 (0.5) [0.5 on inpoisson[4]] -> cell 4, seg 0 (0.5) [0.5 on Ipop[4]], weight: 4.1363508E-4, delay 0.025
        h("Ipop[4] syn_projIinput_ampa[4] = new ampa(0.5)")
        h("inpoisson[4] netConn_projIinput_ampa[4] = new NetCon(m_poisson_input_inpoisson[4], syn_projIinput_ampa[4], 0.0, 0.025, 4.1363508E-4)")  

        # ######################   Projection: projEE
        print("Adding projection: projEE, from Epop to Epop with synapse ampa, 28 connection(s)")

        h("objectvar syn_projEE_ampa[28]")

        h("objectvar netConn_projEE_ampa[28]")

        # Connection 0: cell 0, seg 0 (0.5) [0.5 on Epop[0]] -> cell 4, seg 0 (0.5) [0.5 on Epop[4]], weight: 4.1363508E-4, delay 1.5
        h("Epop[4] syn_projEE_ampa[0] = new ampa(0.5)")
        h("Epop[0] netConn_projEE_ampa[0] = new NetCon(&v(0.5), syn_projEE_ampa[0], 20.0, 1.5, 4.1363508E-4)")  

        # Connection 1: cell 0, seg 0 (0.5) [0.5 on Epop[0]] -> cell 14, seg 0 (0.5) [0.5 on Epop[14]], weight: 4.1363508E-4, delay 1.5
        h("Epop[14] syn_projEE_ampa[1] = new ampa(0.5)")
        h("Epop[0] netConn_projEE_ampa[1] = new NetCon(&v(0.5), syn_projEE_ampa[1], 20.0, 1.5, 4.1363508E-4)")  

        # Connection 2: cell 1, seg 0 (0.5) [0.5 on Epop[1]] -> cell 0, seg 0 (0.5) [0.5 on Epop[0]], weight: 4.1363508E-4, delay 1.5
        h("Epop[0] syn_projEE_ampa[2] = new ampa(0.5)")
        h("Epop[1] netConn_projEE_ampa[2] = new NetCon(&v(0.5), syn_projEE_ampa[2], 20.0, 1.5, 4.1363508E-4)")  

        # Connection 3: cell 2, seg 0 (0.5) [0.5 on Epop[2]] -> cell 2, seg 0 (0.5) [0.5 on Epop[2]], weight: 4.1363508E-4, delay 1.5
        h("Epop[2] syn_projEE_ampa[3] = new ampa(0.5)")
        h("Epop[2] netConn_projEE_ampa[3] = new NetCon(&v(0.5), syn_projEE_ampa[3], 20.0, 1.5, 4.1363508E-4)")  

        # Connection 4: cell 3, seg 0 (0.5) [0.5 on Epop[3]] -> cell 9, seg 0 (0.5) [0.5 on Epop[9]], weight: 4.1363508E-4, delay 1.5
        h("Epop[9] syn_projEE_ampa[4] = new ampa(0.5)")
        h("Epop[3] netConn_projEE_ampa[4] = new NetCon(&v(0.5), syn_projEE_ampa[4], 20.0, 1.5, 4.1363508E-4)")  

        # Connection 5: cell 3, seg 0 (0.5) [0.5 on Epop[3]] -> cell 10, seg 0 (0.5) [0.5 on Epop[10]], weight: 4.1363508E-4, delay 1.5
        h("Epop[10] syn_projEE_ampa[5] = new ampa(0.5)")
        h("Epop[3] netConn_projEE_ampa[5] = new NetCon(&v(0.5), syn_projEE_ampa[5], 20.0, 1.5, 4.1363508E-4)")  

        # Connection 6: cell 3, seg 0 (0.5) [0.5 on Epop[3]] -> cell 18, seg 0 (0.5) [0.5 on Epop[18]], weight: 4.1363508E-4, delay 1.5
        h("Epop[18] syn_projEE_ampa[6] = new ampa(0.5)")
        h("Epop[3] netConn_projEE_ampa[6] = new NetCon(&v(0.5), syn_projEE_ampa[6], 20.0, 1.5, 4.1363508E-4)")  

        # Connection 7: cell 5, seg 0 (0.5) [0.5 on Epop[5]] -> cell 17, seg 0 (0.5) [0.5 on Epop[17]], weight: 4.1363508E-4, delay 1.5
        h("Epop[17] syn_projEE_ampa[7] = new ampa(0.5)")
        h("Epop[5] netConn_projEE_ampa[7] = new NetCon(&v(0.5), syn_projEE_ampa[7], 20.0, 1.5, 4.1363508E-4)")  

        # Connection 8: cell 6, seg 0 (0.5) [0.5 on Epop[6]] -> cell 5, seg 0 (0.5) [0.5 on Epop[5]], weight: 4.1363508E-4, delay 1.5
        h("Epop[5] syn_projEE_ampa[8] = new ampa(0.5)")
        h("Epop[6] netConn_projEE_ampa[8] = new NetCon(&v(0.5), syn_projEE_ampa[8], 20.0, 1.5, 4.1363508E-4)")  

        # Connection 9: cell 6, seg 0 (0.5) [0.5 on Epop[6]] -> cell 7, seg 0 (0.5) [0.5 on Epop[7]], weight: 4.1363508E-4, delay 1.5
        h("Epop[7] syn_projEE_ampa[9] = new ampa(0.5)")
        h("Epop[6] netConn_projEE_ampa[9] = new NetCon(&v(0.5), syn_projEE_ampa[9], 20.0, 1.5, 4.1363508E-4)")  

        # Connection 10: cell 7, seg 0 (0.5) [0.5 on Epop[7]] -> cell 9, seg 0 (0.5) [0.5 on Epop[9]], weight: 4.1363508E-4, delay 1.5
        h("Epop[9] syn_projEE_ampa[10] = new ampa(0.5)")
        h("Epop[7] netConn_projEE_ampa[10] = new NetCon(&v(0.5), syn_projEE_ampa[10], 20.0, 1.5, 4.1363508E-4)")  

        # Connection 11: cell 7, seg 0 (0.5) [0.5 on Epop[7]] -> cell 17, seg 0 (0.5) [0.5 on Epop[17]], weight: 4.1363508E-4, delay 1.5
        h("Epop[17] syn_projEE_ampa[11] = new ampa(0.5)")
        h("Epop[7] netConn_projEE_ampa[11] = new NetCon(&v(0.5), syn_projEE_ampa[11], 20.0, 1.5, 4.1363508E-4)")  

        # Connection 12: cell 10, seg 0 (0.5) [0.5 on Epop[10]] -> cell 18, seg 0 (0.5) [0.5 on Epop[18]], weight: 4.1363508E-4, delay 1.5
        h("Epop[18] syn_projEE_ampa[12] = new ampa(0.5)")
        h("Epop[10] netConn_projEE_ampa[12] = new NetCon(&v(0.5), syn_projEE_ampa[12], 20.0, 1.5, 4.1363508E-4)")  

        # Connection 13: cell 11, seg 0 (0.5) [0.5 on Epop[11]] -> cell 5, seg 0 (0.5) [0.5 on Epop[5]], weight: 4.1363508E-4, delay 1.5
        h("Epop[5] syn_projEE_ampa[13] = new ampa(0.5)")
        h("Epop[11] netConn_projEE_ampa[13] = new NetCon(&v(0.5), syn_projEE_ampa[13], 20.0, 1.5, 4.1363508E-4)")  

        # Connection 14: cell 11, seg 0 (0.5) [0.5 on Epop[11]] -> cell 8, seg 0 (0.5) [0.5 on Epop[8]], weight: 4.1363508E-4, delay 1.5
        h("Epop[8] syn_projEE_ampa[14] = new ampa(0.5)")
        h("Epop[11] netConn_projEE_ampa[14] = new NetCon(&v(0.5), syn_projEE_ampa[14], 20.0, 1.5, 4.1363508E-4)")  

        # Connection 15: cell 11, seg 0 (0.5) [0.5 on Epop[11]] -> cell 9, seg 0 (0.5) [0.5 on Epop[9]], weight: 4.1363508E-4, delay 1.5
        h("Epop[9] syn_projEE_ampa[15] = new ampa(0.5)")
        h("Epop[11] netConn_projEE_ampa[15] = new NetCon(&v(0.5), syn_projEE_ampa[15], 20.0, 1.5, 4.1363508E-4)")  

        # Connection 16: cell 11, seg 0 (0.5) [0.5 on Epop[11]] -> cell 19, seg 0 (0.5) [0.5 on Epop[19]], weight: 4.1363508E-4, delay 1.5
        h("Epop[19] syn_projEE_ampa[16] = new ampa(0.5)")
        h("Epop[11] netConn_projEE_ampa[16] = new NetCon(&v(0.5), syn_projEE_ampa[16], 20.0, 1.5, 4.1363508E-4)")  

        # Connection 17: cell 13, seg 0 (0.5) [0.5 on Epop[13]] -> cell 14, seg 0 (0.5) [0.5 on Epop[14]], weight: 4.1363508E-4, delay 1.5
        h("Epop[14] syn_projEE_ampa[17] = new ampa(0.5)")
        h("Epop[13] netConn_projEE_ampa[17] = new NetCon(&v(0.5), syn_projEE_ampa[17], 20.0, 1.5, 4.1363508E-4)")  

        # Connection 18: cell 15, seg 0 (0.5) [0.5 on Epop[15]] -> cell 10, seg 0 (0.5) [0.5 on Epop[10]], weight: 4.1363508E-4, delay 1.5
        h("Epop[10] syn_projEE_ampa[18] = new ampa(0.5)")
        h("Epop[15] netConn_projEE_ampa[18] = new NetCon(&v(0.5), syn_projEE_ampa[18], 20.0, 1.5, 4.1363508E-4)")  

        # Connection 19: cell 16, seg 0 (0.5) [0.5 on Epop[16]] -> cell 9, seg 0 (0.5) [0.5 on Epop[9]], weight: 4.1363508E-4, delay 1.5
        h("Epop[9] syn_projEE_ampa[19] = new ampa(0.5)")
        h("Epop[16] netConn_projEE_ampa[19] = new NetCon(&v(0.5), syn_projEE_ampa[19], 20.0, 1.5, 4.1363508E-4)")  

        # Connection 20: cell 16, seg 0 (0.5) [0.5 on Epop[16]] -> cell 13, seg 0 (0.5) [0.5 on Epop[13]], weight: 4.1363508E-4, delay 1.5
        h("Epop[13] syn_projEE_ampa[20] = new ampa(0.5)")
        h("Epop[16] netConn_projEE_ampa[20] = new NetCon(&v(0.5), syn_projEE_ampa[20], 20.0, 1.5, 4.1363508E-4)")  

        # Connection 21: cell 17, seg 0 (0.5) [0.5 on Epop[17]] -> cell 5, seg 0 (0.5) [0.5 on Epop[5]], weight: 4.1363508E-4, delay 1.5
        h("Epop[5] syn_projEE_ampa[21] = new ampa(0.5)")
        h("Epop[17] netConn_projEE_ampa[21] = new NetCon(&v(0.5), syn_projEE_ampa[21], 20.0, 1.5, 4.1363508E-4)")  

        # Connection 22: cell 18, seg 0 (0.5) [0.5 on Epop[18]] -> cell 3, seg 0 (0.5) [0.5 on Epop[3]], weight: 4.1363508E-4, delay 1.5
        h("Epop[3] syn_projEE_ampa[22] = new ampa(0.5)")
        h("Epop[18] netConn_projEE_ampa[22] = new NetCon(&v(0.5), syn_projEE_ampa[22], 20.0, 1.5, 4.1363508E-4)")  

        # Connection 23: cell 18, seg 0 (0.5) [0.5 on Epop[18]] -> cell 7, seg 0 (0.5) [0.5 on Epop[7]], weight: 4.1363508E-4, delay 1.5
        h("Epop[7] syn_projEE_ampa[23] = new ampa(0.5)")
        h("Epop[18] netConn_projEE_ampa[23] = new NetCon(&v(0.5), syn_projEE_ampa[23], 20.0, 1.5, 4.1363508E-4)")  

        # Connection 24: cell 18, seg 0 (0.5) [0.5 on Epop[18]] -> cell 11, seg 0 (0.5) [0.5 on Epop[11]], weight: 4.1363508E-4, delay 1.5
        h("Epop[11] syn_projEE_ampa[24] = new ampa(0.5)")
        h("Epop[18] netConn_projEE_ampa[24] = new NetCon(&v(0.5), syn_projEE_ampa[24], 20.0, 1.5, 4.1363508E-4)")  

        # Connection 25: cell 18, seg 0 (0.5) [0.5 on Epop[18]] -> cell 12, seg 0 (0.5) [0.5 on Epop[12]], weight: 4.1363508E-4, delay 1.5
        h("Epop[12] syn_projEE_ampa[25] = new ampa(0.5)")
        h("Epop[18] netConn_projEE_ampa[25] = new NetCon(&v(0.5), syn_projEE_ampa[25], 20.0, 1.5, 4.1363508E-4)")  

        # Connection 26: cell 18, seg 0 (0.5) [0.5 on Epop[18]] -> cell 14, seg 0 (0.5) [0.5 on Epop[14]], weight: 4.1363508E-4, delay 1.5
        h("Epop[14] syn_projEE_ampa[26] = new ampa(0.5)")
        h("Epop[18] netConn_projEE_ampa[26] = new NetCon(&v(0.5), syn_projEE_ampa[26], 20.0, 1.5, 4.1363508E-4)")  

        # Connection 27: cell 19, seg 0 (0.5) [0.5 on Epop[19]] -> cell 7, seg 0 (0.5) [0.5 on Epop[7]], weight: 4.1363508E-4, delay 1.5
        h("Epop[7] syn_projEE_ampa[27] = new ampa(0.5)")
        h("Epop[19] netConn_projEE_ampa[27] = new NetCon(&v(0.5), syn_projEE_ampa[27], 20.0, 1.5, 4.1363508E-4)")  

        # ######################   Projection: projEI
        print("Adding projection: projEI, from Epop to Ipop with synapse ampa, 12 connection(s)")

        h("objectvar syn_projEI_ampa[12]")

        h("objectvar netConn_projEI_ampa[12]")

        # Connection 0: cell 0, seg 0 (0.5) [0.5 on Epop[0]] -> cell 2, seg 0 (0.5) [0.5 on Ipop[2]], weight: 4.1363508E-4, delay 1.5
        h("Ipop[2] syn_projEI_ampa[0] = new ampa(0.5)")
        h("Epop[0] netConn_projEI_ampa[0] = new NetCon(&v(0.5), syn_projEI_ampa[0], 20.0, 1.5, 4.1363508E-4)")  

        # Connection 1: cell 6, seg 0 (0.5) [0.5 on Epop[6]] -> cell 2, seg 0 (0.5) [0.5 on Ipop[2]], weight: 4.1363508E-4, delay 1.5
        h("Ipop[2] syn_projEI_ampa[1] = new ampa(0.5)")
        h("Epop[6] netConn_projEI_ampa[1] = new NetCon(&v(0.5), syn_projEI_ampa[1], 20.0, 1.5, 4.1363508E-4)")  

        # Connection 2: cell 7, seg 0 (0.5) [0.5 on Epop[7]] -> cell 1, seg 0 (0.5) [0.5 on Ipop[1]], weight: 4.1363508E-4, delay 1.5
        h("Ipop[1] syn_projEI_ampa[2] = new ampa(0.5)")
        h("Epop[7] netConn_projEI_ampa[2] = new NetCon(&v(0.5), syn_projEI_ampa[2], 20.0, 1.5, 4.1363508E-4)")  

        # Connection 3: cell 7, seg 0 (0.5) [0.5 on Epop[7]] -> cell 2, seg 0 (0.5) [0.5 on Ipop[2]], weight: 4.1363508E-4, delay 1.5
        h("Ipop[2] syn_projEI_ampa[3] = new ampa(0.5)")
        h("Epop[7] netConn_projEI_ampa[3] = new NetCon(&v(0.5), syn_projEI_ampa[3], 20.0, 1.5, 4.1363508E-4)")  

        # Connection 4: cell 10, seg 0 (0.5) [0.5 on Epop[10]] -> cell 0, seg 0 (0.5) [0.5 on Ipop[0]], weight: 4.1363508E-4, delay 1.5
        h("Ipop[0] syn_projEI_ampa[4] = new ampa(0.5)")
        h("Epop[10] netConn_projEI_ampa[4] = new NetCon(&v(0.5), syn_projEI_ampa[4], 20.0, 1.5, 4.1363508E-4)")  

        # Connection 5: cell 11, seg 0 (0.5) [0.5 on Epop[11]] -> cell 0, seg 0 (0.5) [0.5 on Ipop[0]], weight: 4.1363508E-4, delay 1.5
        h("Ipop[0] syn_projEI_ampa[5] = new ampa(0.5)")
        h("Epop[11] netConn_projEI_ampa[5] = new NetCon(&v(0.5), syn_projEI_ampa[5], 20.0, 1.5, 4.1363508E-4)")  

        # Connection 6: cell 11, seg 0 (0.5) [0.5 on Epop[11]] -> cell 4, seg 0 (0.5) [0.5 on Ipop[4]], weight: 4.1363508E-4, delay 1.5
        h("Ipop[4] syn_projEI_ampa[6] = new ampa(0.5)")
        h("Epop[11] netConn_projEI_ampa[6] = new NetCon(&v(0.5), syn_projEI_ampa[6], 20.0, 1.5, 4.1363508E-4)")  

        # Connection 7: cell 12, seg 0 (0.5) [0.5 on Epop[12]] -> cell 0, seg 0 (0.5) [0.5 on Ipop[0]], weight: 4.1363508E-4, delay 1.5
        h("Ipop[0] syn_projEI_ampa[7] = new ampa(0.5)")
        h("Epop[12] netConn_projEI_ampa[7] = new NetCon(&v(0.5), syn_projEI_ampa[7], 20.0, 1.5, 4.1363508E-4)")  

        # Connection 8: cell 13, seg 0 (0.5) [0.5 on Epop[13]] -> cell 2, seg 0 (0.5) [0.5 on Ipop[2]], weight: 4.1363508E-4, delay 1.5
        h("Ipop[2] syn_projEI_ampa[8] = new ampa(0.5)")
        h("Epop[13] netConn_projEI_ampa[8] = new NetCon(&v(0.5), syn_projEI_ampa[8], 20.0, 1.5, 4.1363508E-4)")  

        # Connection 9: cell 14, seg 0 (0.5) [0.5 on Epop[14]] -> cell 4, seg 0 (0.5) [0.5 on Ipop[4]], weight: 4.1363508E-4, delay 1.5
        h("Ipop[4] syn_projEI_ampa[9] = new ampa(0.5)")
        h("Epop[14] netConn_projEI_ampa[9] = new NetCon(&v(0.5), syn_projEI_ampa[9], 20.0, 1.5, 4.1363508E-4)")  

        # Connection 10: cell 15, seg 0 (0.5) [0.5 on Epop[15]] -> cell 3, seg 0 (0.5) [0.5 on Ipop[3]], weight: 4.1363508E-4, delay 1.5
        h("Ipop[3] syn_projEI_ampa[10] = new ampa(0.5)")
        h("Epop[15] netConn_projEI_ampa[10] = new NetCon(&v(0.5), syn_projEI_ampa[10], 20.0, 1.5, 4.1363508E-4)")  

        # Connection 11: cell 16, seg 0 (0.5) [0.5 on Epop[16]] -> cell 4, seg 0 (0.5) [0.5 on Ipop[4]], weight: 4.1363508E-4, delay 1.5
        h("Ipop[4] syn_projEI_ampa[11] = new ampa(0.5)")
        h("Epop[16] netConn_projEI_ampa[11] = new NetCon(&v(0.5), syn_projEI_ampa[11], 20.0, 1.5, 4.1363508E-4)")  

        # ######################   Projection: projIE
        print("Adding projection: projIE, from Ipop to Epop with synapse gaba, 4 connection(s)")

        h("objectvar syn_projIE_gaba[4]")

        h("objectvar netConn_projIE_gaba[4]")

        # Connection 0: cell 0, seg 0 (0.5) [0.5 on Ipop[0]] -> cell 18, seg 0 (0.5) [0.5 on Epop[18]], weight: -0.0016545403, delay 1.5
        h("Epop[18] syn_projIE_gaba[0] = new gaba(0.5)")
        h("Ipop[0] netConn_projIE_gaba[0] = new NetCon(&v(0.5), syn_projIE_gaba[0], 20.0, 1.5, -0.0016545403)")  

        # Connection 1: cell 1, seg 0 (0.5) [0.5 on Ipop[1]] -> cell 2, seg 0 (0.5) [0.5 on Epop[2]], weight: -0.0016545403, delay 1.5
        h("Epop[2] syn_projIE_gaba[1] = new gaba(0.5)")
        h("Ipop[1] netConn_projIE_gaba[1] = new NetCon(&v(0.5), syn_projIE_gaba[1], 20.0, 1.5, -0.0016545403)")  

        # Connection 2: cell 1, seg 0 (0.5) [0.5 on Ipop[1]] -> cell 3, seg 0 (0.5) [0.5 on Epop[3]], weight: -0.0016545403, delay 1.5
        h("Epop[3] syn_projIE_gaba[2] = new gaba(0.5)")
        h("Ipop[1] netConn_projIE_gaba[2] = new NetCon(&v(0.5), syn_projIE_gaba[2], 20.0, 1.5, -0.0016545403)")  

        # Connection 3: cell 4, seg 0 (0.5) [0.5 on Ipop[4]] -> cell 6, seg 0 (0.5) [0.5 on Epop[6]], weight: -0.0016545403, delay 1.5
        h("Epop[6] syn_projIE_gaba[3] = new gaba(0.5)")
        h("Ipop[4] netConn_projIE_gaba[3] = new NetCon(&v(0.5), syn_projIE_gaba[3], 20.0, 1.5, -0.0016545403)")  

        # ######################   Projection: projII
        print("Adding projection: projII, from Ipop to Ipop with synapse gaba, 2 connection(s)")

        h("objectvar syn_projII_gaba[2]")

        h("objectvar netConn_projII_gaba[2]")

        # Connection 0: cell 0, seg 0 (0.5) [0.5 on Ipop[0]] -> cell 4, seg 0 (0.5) [0.5 on Ipop[4]], weight: -0.0016545403, delay 1.5
        h("Ipop[4] syn_projII_gaba[0] = new gaba(0.5)")
        h("Ipop[0] netConn_projII_gaba[0] = new NetCon(&v(0.5), syn_projII_gaba[0], 20.0, 1.5, -0.0016545403)")  

        # Connection 1: cell 2, seg 0 (0.5) [0.5 on Ipop[2]] -> cell 4, seg 0 (0.5) [0.5 on Ipop[4]], weight: -0.0016545403, delay 1.5
        h("Ipop[4] syn_projII_gaba[1] = new gaba(0.5)")
        h("Ipop[2] netConn_projII_gaba[1] = new NetCon(&v(0.5), syn_projII_gaba[1], 20.0, 1.5, -0.0016545403)")  

        trec = h.Vector()
        trec.record(h._ref_t)

        h.tstop = tstop

        h.dt = dt

        h.steps_per_ms = 1/h.dt



        # ######################   File to save: SimExample7.Ipop.spikes (SimExample7_Ipop_spikes)
        h(' objectvar spiketimes_SimExample7_Ipop_spikes, t_spiketimes_SimExample7_Ipop_spikes ')
        h(' { spiketimes_SimExample7_Ipop_spikes = new Vector() } ')
        h(' { t_spiketimes_SimExample7_Ipop_spikes = new Vector() } ')
        h(' objref netConnSpike_SimExample7_Ipop_spikes, nil ')
        # Column: Ipop/0/ifcell (0) Ipop[0]
        h(' Ipop[0] { netConnSpike_SimExample7_Ipop_spikes = new NetCon(&v(0.5), nil, 20.0, 0, 1) } ')
        h(' { netConnSpike_SimExample7_Ipop_spikes.record(t_spiketimes_SimExample7_Ipop_spikes, spiketimes_SimExample7_Ipop_spikes, 0) } ')
        # Column: Ipop/1/ifcell (1) Ipop[1]
        h(' Ipop[1] { netConnSpike_SimExample7_Ipop_spikes = new NetCon(&v(0.5), nil, 20.0, 0, 1) } ')
        h(' { netConnSpike_SimExample7_Ipop_spikes.record(t_spiketimes_SimExample7_Ipop_spikes, spiketimes_SimExample7_Ipop_spikes, 1) } ')
        # Column: Ipop/2/ifcell (2) Ipop[2]
        h(' Ipop[2] { netConnSpike_SimExample7_Ipop_spikes = new NetCon(&v(0.5), nil, 20.0, 0, 1) } ')
        h(' { netConnSpike_SimExample7_Ipop_spikes.record(t_spiketimes_SimExample7_Ipop_spikes, spiketimes_SimExample7_Ipop_spikes, 2) } ')
        # Column: Ipop/3/ifcell (3) Ipop[3]
        h(' Ipop[3] { netConnSpike_SimExample7_Ipop_spikes = new NetCon(&v(0.5), nil, 20.0, 0, 1) } ')
        h(' { netConnSpike_SimExample7_Ipop_spikes.record(t_spiketimes_SimExample7_Ipop_spikes, spiketimes_SimExample7_Ipop_spikes, 3) } ')
        # Column: Ipop/4/ifcell (4) Ipop[4]
        h(' Ipop[4] { netConnSpike_SimExample7_Ipop_spikes = new NetCon(&v(0.5), nil, 20.0, 0, 1) } ')
        h(' { netConnSpike_SimExample7_Ipop_spikes.record(t_spiketimes_SimExample7_Ipop_spikes, spiketimes_SimExample7_Ipop_spikes, 4) } ')

        # ######################   File to save: SimExample7.Ipop.v.dat (SimExample7_Ipop_v_dat)
        # Column: Ipop/0/ifcell/v
        h(' objectvar v_Ipop_0_ifcell_v_SimExample7_Ipop_v_dat ')
        h(' { v_Ipop_0_ifcell_v_SimExample7_Ipop_v_dat = new Vector() } ')
        h(' { v_Ipop_0_ifcell_v_SimExample7_Ipop_v_dat.record(&Ipop[0].v(0.5)) } ')
        h.v_Ipop_0_ifcell_v_SimExample7_Ipop_v_dat.resize((h.tstop * h.steps_per_ms) + 1)
        # Column: Ipop/1/ifcell/v
        h(' objectvar v_Ipop_1_ifcell_v_SimExample7_Ipop_v_dat ')
        h(' { v_Ipop_1_ifcell_v_SimExample7_Ipop_v_dat = new Vector() } ')
        h(' { v_Ipop_1_ifcell_v_SimExample7_Ipop_v_dat.record(&Ipop[1].v(0.5)) } ')
        h.v_Ipop_1_ifcell_v_SimExample7_Ipop_v_dat.resize((h.tstop * h.steps_per_ms) + 1)

        # ######################   File to save: SimExample7.inpoisson.spikes (SimExample7_inpoisson_spikes)
        h(' objectvar spiketimes_SimExample7_inpoisson_spikes, t_spiketimes_SimExample7_inpoisson_spikes ')
        h(' { spiketimes_SimExample7_inpoisson_spikes = new Vector() } ')
        h(' { t_spiketimes_SimExample7_inpoisson_spikes = new Vector() } ')
        h(' objref netConnSpike_SimExample7_inpoisson_spikes, nil ')
        # It's a spike source, will listen to m_poisson_input_inpoisson...
        h(' netConnSpike_SimExample7_inpoisson_spikes = new NetCon(m_poisson_input_inpoisson[0], nil) ')
        h(' { netConnSpike_SimExample7_inpoisson_spikes.record(t_spiketimes_SimExample7_inpoisson_spikes, spiketimes_SimExample7_inpoisson_spikes, 0) } ')
        # It's a spike source, will listen to m_poisson_input_inpoisson...
        h(' netConnSpike_SimExample7_inpoisson_spikes = new NetCon(m_poisson_input_inpoisson[1], nil) ')
        h(' { netConnSpike_SimExample7_inpoisson_spikes.record(t_spiketimes_SimExample7_inpoisson_spikes, spiketimes_SimExample7_inpoisson_spikes, 1) } ')
        # It's a spike source, will listen to m_poisson_input_inpoisson...
        h(' netConnSpike_SimExample7_inpoisson_spikes = new NetCon(m_poisson_input_inpoisson[2], nil) ')
        h(' { netConnSpike_SimExample7_inpoisson_spikes.record(t_spiketimes_SimExample7_inpoisson_spikes, spiketimes_SimExample7_inpoisson_spikes, 2) } ')
        # It's a spike source, will listen to m_poisson_input_inpoisson...
        h(' netConnSpike_SimExample7_inpoisson_spikes = new NetCon(m_poisson_input_inpoisson[3], nil) ')
        h(' { netConnSpike_SimExample7_inpoisson_spikes.record(t_spiketimes_SimExample7_inpoisson_spikes, spiketimes_SimExample7_inpoisson_spikes, 3) } ')
        # It's a spike source, will listen to m_poisson_input_inpoisson...
        h(' netConnSpike_SimExample7_inpoisson_spikes = new NetCon(m_poisson_input_inpoisson[4], nil) ')
        h(' { netConnSpike_SimExample7_inpoisson_spikes.record(t_spiketimes_SimExample7_inpoisson_spikes, spiketimes_SimExample7_inpoisson_spikes, 4) } ')

        # ######################   File to save: SimExample7.Epop.v.dat (SimExample7_Epop_v_dat)
        # Column: Epop/0/ifcell/v
        h(' objectvar v_Epop_0_ifcell_v_SimExample7_Epop_v_dat ')
        h(' { v_Epop_0_ifcell_v_SimExample7_Epop_v_dat = new Vector() } ')
        h(' { v_Epop_0_ifcell_v_SimExample7_Epop_v_dat.record(&Epop[0].v(0.5)) } ')
        h.v_Epop_0_ifcell_v_SimExample7_Epop_v_dat.resize((h.tstop * h.steps_per_ms) + 1)
        # Column: Epop/1/ifcell/v
        h(' objectvar v_Epop_1_ifcell_v_SimExample7_Epop_v_dat ')
        h(' { v_Epop_1_ifcell_v_SimExample7_Epop_v_dat = new Vector() } ')
        h(' { v_Epop_1_ifcell_v_SimExample7_Epop_v_dat.record(&Epop[1].v(0.5)) } ')
        h.v_Epop_1_ifcell_v_SimExample7_Epop_v_dat.resize((h.tstop * h.steps_per_ms) + 1)

        # ######################   File to save: time.dat (time)
        # Column: time
        h(' objectvar v_time ')
        h(' { v_time = new Vector() } ')
        h(' { v_time.record(&t) } ')
        h.v_time.resize((h.tstop * h.steps_per_ms) + 1)

        # ######################   File to save: SimExample7.Epop.spikes (SimExample7_Epop_spikes)
        h(' objectvar spiketimes_SimExample7_Epop_spikes, t_spiketimes_SimExample7_Epop_spikes ')
        h(' { spiketimes_SimExample7_Epop_spikes = new Vector() } ')
        h(' { t_spiketimes_SimExample7_Epop_spikes = new Vector() } ')
        h(' objref netConnSpike_SimExample7_Epop_spikes, nil ')
        # Column: Epop/0/ifcell (0) Epop[0]
        h(' Epop[0] { netConnSpike_SimExample7_Epop_spikes = new NetCon(&v(0.5), nil, 20.0, 0, 1) } ')
        h(' { netConnSpike_SimExample7_Epop_spikes.record(t_spiketimes_SimExample7_Epop_spikes, spiketimes_SimExample7_Epop_spikes, 0) } ')
        # Column: Epop/1/ifcell (1) Epop[1]
        h(' Epop[1] { netConnSpike_SimExample7_Epop_spikes = new NetCon(&v(0.5), nil, 20.0, 0, 1) } ')
        h(' { netConnSpike_SimExample7_Epop_spikes.record(t_spiketimes_SimExample7_Epop_spikes, spiketimes_SimExample7_Epop_spikes, 1) } ')
        # Column: Epop/2/ifcell (2) Epop[2]
        h(' Epop[2] { netConnSpike_SimExample7_Epop_spikes = new NetCon(&v(0.5), nil, 20.0, 0, 1) } ')
        h(' { netConnSpike_SimExample7_Epop_spikes.record(t_spiketimes_SimExample7_Epop_spikes, spiketimes_SimExample7_Epop_spikes, 2) } ')
        # Column: Epop/3/ifcell (3) Epop[3]
        h(' Epop[3] { netConnSpike_SimExample7_Epop_spikes = new NetCon(&v(0.5), nil, 20.0, 0, 1) } ')
        h(' { netConnSpike_SimExample7_Epop_spikes.record(t_spiketimes_SimExample7_Epop_spikes, spiketimes_SimExample7_Epop_spikes, 3) } ')
        # Column: Epop/4/ifcell (4) Epop[4]
        h(' Epop[4] { netConnSpike_SimExample7_Epop_spikes = new NetCon(&v(0.5), nil, 20.0, 0, 1) } ')
        h(' { netConnSpike_SimExample7_Epop_spikes.record(t_spiketimes_SimExample7_Epop_spikes, spiketimes_SimExample7_Epop_spikes, 4) } ')
        # Column: Epop/5/ifcell (5) Epop[5]
        h(' Epop[5] { netConnSpike_SimExample7_Epop_spikes = new NetCon(&v(0.5), nil, 20.0, 0, 1) } ')
        h(' { netConnSpike_SimExample7_Epop_spikes.record(t_spiketimes_SimExample7_Epop_spikes, spiketimes_SimExample7_Epop_spikes, 5) } ')
        # Column: Epop/6/ifcell (6) Epop[6]
        h(' Epop[6] { netConnSpike_SimExample7_Epop_spikes = new NetCon(&v(0.5), nil, 20.0, 0, 1) } ')
        h(' { netConnSpike_SimExample7_Epop_spikes.record(t_spiketimes_SimExample7_Epop_spikes, spiketimes_SimExample7_Epop_spikes, 6) } ')
        # Column: Epop/7/ifcell (7) Epop[7]
        h(' Epop[7] { netConnSpike_SimExample7_Epop_spikes = new NetCon(&v(0.5), nil, 20.0, 0, 1) } ')
        h(' { netConnSpike_SimExample7_Epop_spikes.record(t_spiketimes_SimExample7_Epop_spikes, spiketimes_SimExample7_Epop_spikes, 7) } ')
        # Column: Epop/8/ifcell (8) Epop[8]
        h(' Epop[8] { netConnSpike_SimExample7_Epop_spikes = new NetCon(&v(0.5), nil, 20.0, 0, 1) } ')
        h(' { netConnSpike_SimExample7_Epop_spikes.record(t_spiketimes_SimExample7_Epop_spikes, spiketimes_SimExample7_Epop_spikes, 8) } ')
        # Column: Epop/9/ifcell (9) Epop[9]
        h(' Epop[9] { netConnSpike_SimExample7_Epop_spikes = new NetCon(&v(0.5), nil, 20.0, 0, 1) } ')
        h(' { netConnSpike_SimExample7_Epop_spikes.record(t_spiketimes_SimExample7_Epop_spikes, spiketimes_SimExample7_Epop_spikes, 9) } ')
        # Column: Epop/10/ifcell (10) Epop[10]
        h(' Epop[10] { netConnSpike_SimExample7_Epop_spikes = new NetCon(&v(0.5), nil, 20.0, 0, 1) } ')
        h(' { netConnSpike_SimExample7_Epop_spikes.record(t_spiketimes_SimExample7_Epop_spikes, spiketimes_SimExample7_Epop_spikes, 10) } ')
        # Column: Epop/11/ifcell (11) Epop[11]
        h(' Epop[11] { netConnSpike_SimExample7_Epop_spikes = new NetCon(&v(0.5), nil, 20.0, 0, 1) } ')
        h(' { netConnSpike_SimExample7_Epop_spikes.record(t_spiketimes_SimExample7_Epop_spikes, spiketimes_SimExample7_Epop_spikes, 11) } ')
        # Column: Epop/12/ifcell (12) Epop[12]
        h(' Epop[12] { netConnSpike_SimExample7_Epop_spikes = new NetCon(&v(0.5), nil, 20.0, 0, 1) } ')
        h(' { netConnSpike_SimExample7_Epop_spikes.record(t_spiketimes_SimExample7_Epop_spikes, spiketimes_SimExample7_Epop_spikes, 12) } ')
        # Column: Epop/13/ifcell (13) Epop[13]
        h(' Epop[13] { netConnSpike_SimExample7_Epop_spikes = new NetCon(&v(0.5), nil, 20.0, 0, 1) } ')
        h(' { netConnSpike_SimExample7_Epop_spikes.record(t_spiketimes_SimExample7_Epop_spikes, spiketimes_SimExample7_Epop_spikes, 13) } ')
        # Column: Epop/14/ifcell (14) Epop[14]
        h(' Epop[14] { netConnSpike_SimExample7_Epop_spikes = new NetCon(&v(0.5), nil, 20.0, 0, 1) } ')
        h(' { netConnSpike_SimExample7_Epop_spikes.record(t_spiketimes_SimExample7_Epop_spikes, spiketimes_SimExample7_Epop_spikes, 14) } ')
        # Column: Epop/15/ifcell (15) Epop[15]
        h(' Epop[15] { netConnSpike_SimExample7_Epop_spikes = new NetCon(&v(0.5), nil, 20.0, 0, 1) } ')
        h(' { netConnSpike_SimExample7_Epop_spikes.record(t_spiketimes_SimExample7_Epop_spikes, spiketimes_SimExample7_Epop_spikes, 15) } ')
        # Column: Epop/16/ifcell (16) Epop[16]
        h(' Epop[16] { netConnSpike_SimExample7_Epop_spikes = new NetCon(&v(0.5), nil, 20.0, 0, 1) } ')
        h(' { netConnSpike_SimExample7_Epop_spikes.record(t_spiketimes_SimExample7_Epop_spikes, spiketimes_SimExample7_Epop_spikes, 16) } ')
        # Column: Epop/17/ifcell (17) Epop[17]
        h(' Epop[17] { netConnSpike_SimExample7_Epop_spikes = new NetCon(&v(0.5), nil, 20.0, 0, 1) } ')
        h(' { netConnSpike_SimExample7_Epop_spikes.record(t_spiketimes_SimExample7_Epop_spikes, spiketimes_SimExample7_Epop_spikes, 17) } ')
        # Column: Epop/18/ifcell (18) Epop[18]
        h(' Epop[18] { netConnSpike_SimExample7_Epop_spikes = new NetCon(&v(0.5), nil, 20.0, 0, 1) } ')
        h(' { netConnSpike_SimExample7_Epop_spikes.record(t_spiketimes_SimExample7_Epop_spikes, spiketimes_SimExample7_Epop_spikes, 18) } ')
        # Column: Epop/19/ifcell (19) Epop[19]
        h(' Epop[19] { netConnSpike_SimExample7_Epop_spikes = new NetCon(&v(0.5), nil, 20.0, 0, 1) } ')
        h(' { netConnSpike_SimExample7_Epop_spikes.record(t_spiketimes_SimExample7_Epop_spikes, spiketimes_SimExample7_Epop_spikes, 19) } ')

        # ######################   File to save: SimExample7.expoisson.spikes (SimExample7_expoisson_spikes)
        h(' objectvar spiketimes_SimExample7_expoisson_spikes, t_spiketimes_SimExample7_expoisson_spikes ')
        h(' { spiketimes_SimExample7_expoisson_spikes = new Vector() } ')
        h(' { t_spiketimes_SimExample7_expoisson_spikes = new Vector() } ')
        h(' objref netConnSpike_SimExample7_expoisson_spikes, nil ')
        # It's a spike source, will listen to m_poisson_input_expoisson...
        h(' netConnSpike_SimExample7_expoisson_spikes = new NetCon(m_poisson_input_expoisson[0], nil) ')
        h(' { netConnSpike_SimExample7_expoisson_spikes.record(t_spiketimes_SimExample7_expoisson_spikes, spiketimes_SimExample7_expoisson_spikes, 0) } ')
        # It's a spike source, will listen to m_poisson_input_expoisson...
        h(' netConnSpike_SimExample7_expoisson_spikes = new NetCon(m_poisson_input_expoisson[1], nil) ')
        h(' { netConnSpike_SimExample7_expoisson_spikes.record(t_spiketimes_SimExample7_expoisson_spikes, spiketimes_SimExample7_expoisson_spikes, 1) } ')
        # It's a spike source, will listen to m_poisson_input_expoisson...
        h(' netConnSpike_SimExample7_expoisson_spikes = new NetCon(m_poisson_input_expoisson[2], nil) ')
        h(' { netConnSpike_SimExample7_expoisson_spikes.record(t_spiketimes_SimExample7_expoisson_spikes, spiketimes_SimExample7_expoisson_spikes, 2) } ')
        # It's a spike source, will listen to m_poisson_input_expoisson...
        h(' netConnSpike_SimExample7_expoisson_spikes = new NetCon(m_poisson_input_expoisson[3], nil) ')
        h(' { netConnSpike_SimExample7_expoisson_spikes.record(t_spiketimes_SimExample7_expoisson_spikes, spiketimes_SimExample7_expoisson_spikes, 3) } ')
        # It's a spike source, will listen to m_poisson_input_expoisson...
        h(' netConnSpike_SimExample7_expoisson_spikes = new NetCon(m_poisson_input_expoisson[4], nil) ')
        h(' { netConnSpike_SimExample7_expoisson_spikes.record(t_spiketimes_SimExample7_expoisson_spikes, spiketimes_SimExample7_expoisson_spikes, 4) } ')

        self.initialized = False

        self.sim_end = -1 # will be overwritten

        setup_end = time.time()
        self.setup_time = setup_end - self.setup_start
        print("Setting up the network to simulate took %f seconds"%(self.setup_time))

    def run(self):

        self.initialized = True
        sim_start = time.time()
        print("Running a simulation of %sms (dt = %sms; seed=%s)" % (h.tstop, h.dt, self.seed))

        try:
            h.run()
        except Exception as e:
            print("Exception running NEURON: %s" % (e))
            quit()


        self.sim_end = time.time()
        self.sim_time = self.sim_end - sim_start
        print("Finished NEURON simulation in %f seconds (%f mins)..."%(self.sim_time, self.sim_time/60.0))

        try:
            self.save_results()
        except Exception as e:
            print("Exception saving results of NEURON simulation: %s" % (e))
            quit()


    def advance(self):

        if not self.initialized:
            h.finitialize()
            self.initialized = True

        h.fadvance()


    ###############################################################################
    # Hash function to use in generation of random value
    # This is copied from NetPyNE: https://github.com/Neurosim-lab/netpyne/blob/master/netpyne/simFuncs.py
    ###############################################################################
    def _id32 (self,obj): 
        return int(hashlib.md5(obj.encode('utf-8')).hexdigest()[0:8],16)  # convert 8 first chars of md5 hash in base 16 to int


    ###############################################################################
    # Initialize the stim randomizer
    # This is copied from NetPyNE: https://github.com/Neurosim-lab/netpyne/blob/master/netpyne/simFuncs.py
    ###############################################################################
    def _init_stim_randomizer(self,rand, stimType, gid, seed): 
        #print("INIT STIM  %s; %s; %s; %s"%(rand, stimType, gid, seed))
        rand.Random123(self._id32(stimType), gid, seed)


    def save_results(self):

        print("Saving results at t=%s..."%h.t)

        if self.sim_end < 0: self.sim_end = time.time()


        # ######################   File to save: time.dat (time)
        py_v_time = [ t/1000 for t in h.v_time.to_python() ]  # Convert to Python list for speed...

        f_time_f2 = open('time.dat', 'w')
        num_points = len(py_v_time)  # Simulation may have been stopped before tstop...

        for i in range(num_points):
            f_time_f2.write('%f'% py_v_time[i])  # Save in SI units...
        f_time_f2.close()
        print("Saved data to: time.dat")

        # ######################   File to save: SimExample7.Ipop.spikes (SimExample7_Ipop_spikes)

        f_SimExample7_Ipop_spikes_f2 = open('SimExample7.Ipop.spikes', 'w')
        h(' objref netConnSpike_SimExample7_Ipop_spikes ')
        spike_ids = h.spiketimes_SimExample7_Ipop_spikes.to_python()  
        spike_times = h.t_spiketimes_SimExample7_Ipop_spikes.to_python()
        for i, id in enumerate(spike_ids):
            # Saving in format: ID_TIME
            f_SimExample7_Ipop_spikes_f2.write("%i\t%s\n"%(id,spike_times[i]/1000.0))
        f_SimExample7_Ipop_spikes_f2.close()
        print("Saved data to: SimExample7.Ipop.spikes")

        # ######################   File to save: SimExample7.Ipop.v.dat (SimExample7_Ipop_v_dat)
        py_v_Ipop_0_ifcell_v_SimExample7_Ipop_v_dat = [ float(x  / 1000.0) for x in h.v_Ipop_0_ifcell_v_SimExample7_Ipop_v_dat.to_python() ]  # Convert to Python list for speed, variable has dim: voltage
        py_v_Ipop_1_ifcell_v_SimExample7_Ipop_v_dat = [ float(x  / 1000.0) for x in h.v_Ipop_1_ifcell_v_SimExample7_Ipop_v_dat.to_python() ]  # Convert to Python list for speed, variable has dim: voltage

        f_SimExample7_Ipop_v_dat_f2 = open('SimExample7.Ipop.v.dat', 'w')
        num_points = len(py_v_time)  # Simulation may have been stopped before tstop...

        for i in range(num_points):
            f_SimExample7_Ipop_v_dat_f2.write('%e\t%e\t%e\t\n' % (py_v_time[i], py_v_Ipop_0_ifcell_v_SimExample7_Ipop_v_dat[i], py_v_Ipop_1_ifcell_v_SimExample7_Ipop_v_dat[i], ))
        f_SimExample7_Ipop_v_dat_f2.close()
        print("Saved data to: SimExample7.Ipop.v.dat")

        # ######################   File to save: SimExample7.inpoisson.spikes (SimExample7_inpoisson_spikes)

        f_SimExample7_inpoisson_spikes_f2 = open('SimExample7.inpoisson.spikes', 'w')
        h(' objref netConnSpike_SimExample7_inpoisson_spikes ')
        spike_ids = h.spiketimes_SimExample7_inpoisson_spikes.to_python()  
        spike_times = h.t_spiketimes_SimExample7_inpoisson_spikes.to_python()
        for i, id in enumerate(spike_ids):
            # Saving in format: ID_TIME
            f_SimExample7_inpoisson_spikes_f2.write("%i\t%s\n"%(id,spike_times[i]/1000.0))
        f_SimExample7_inpoisson_spikes_f2.close()
        print("Saved data to: SimExample7.inpoisson.spikes")

        # ######################   File to save: SimExample7.Epop.v.dat (SimExample7_Epop_v_dat)
        py_v_Epop_0_ifcell_v_SimExample7_Epop_v_dat = [ float(x  / 1000.0) for x in h.v_Epop_0_ifcell_v_SimExample7_Epop_v_dat.to_python() ]  # Convert to Python list for speed, variable has dim: voltage
        py_v_Epop_1_ifcell_v_SimExample7_Epop_v_dat = [ float(x  / 1000.0) for x in h.v_Epop_1_ifcell_v_SimExample7_Epop_v_dat.to_python() ]  # Convert to Python list for speed, variable has dim: voltage

        f_SimExample7_Epop_v_dat_f2 = open('SimExample7.Epop.v.dat', 'w')
        num_points = len(py_v_time)  # Simulation may have been stopped before tstop...

        for i in range(num_points):
            f_SimExample7_Epop_v_dat_f2.write('%e\t%e\t%e\t\n' % (py_v_time[i], py_v_Epop_0_ifcell_v_SimExample7_Epop_v_dat[i], py_v_Epop_1_ifcell_v_SimExample7_Epop_v_dat[i], ))
        f_SimExample7_Epop_v_dat_f2.close()
        print("Saved data to: SimExample7.Epop.v.dat")

        # ######################   File to save: SimExample7.Epop.spikes (SimExample7_Epop_spikes)

        f_SimExample7_Epop_spikes_f2 = open('SimExample7.Epop.spikes', 'w')
        h(' objref netConnSpike_SimExample7_Epop_spikes ')
        spike_ids = h.spiketimes_SimExample7_Epop_spikes.to_python()  
        spike_times = h.t_spiketimes_SimExample7_Epop_spikes.to_python()
        for i, id in enumerate(spike_ids):
            # Saving in format: ID_TIME
            f_SimExample7_Epop_spikes_f2.write("%i\t%s\n"%(id,spike_times[i]/1000.0))
        f_SimExample7_Epop_spikes_f2.close()
        print("Saved data to: SimExample7.Epop.spikes")

        # ######################   File to save: SimExample7.expoisson.spikes (SimExample7_expoisson_spikes)

        f_SimExample7_expoisson_spikes_f2 = open('SimExample7.expoisson.spikes', 'w')
        h(' objref netConnSpike_SimExample7_expoisson_spikes ')
        spike_ids = h.spiketimes_SimExample7_expoisson_spikes.to_python()  
        spike_times = h.t_spiketimes_SimExample7_expoisson_spikes.to_python()
        for i, id in enumerate(spike_ids):
            # Saving in format: ID_TIME
            f_SimExample7_expoisson_spikes_f2.write("%i\t%s\n"%(id,spike_times[i]/1000.0))
        f_SimExample7_expoisson_spikes_f2.close()
        print("Saved data to: SimExample7.expoisson.spikes")

        save_end = time.time()
        save_time = save_end - self.sim_end
        print("Finished saving results in %f seconds"%(save_time))

        self.report_file.write('StartTime=%s\n'%datetime.datetime.fromtimestamp(self.setup_start).strftime('%Y-%m-%d %H:%M:%S'))
        self.report_file.write('SetupTime=%s\n'%self.setup_time)
        self.report_file.write('RealSimulationTime=%s\n'%self.sim_time)
        self.report_file.write('SimulationSaveTime=%s\n'%save_time)
        self.report_file.close()

        print("Saving report of simulation to %s"%('report.SimExample7.txt'))

        print("Done")

        quit()


if __name__ == '__main__':

    ns = NeuronSimulation(tstop=1000.0, dt=0.025, seed=123)

    ns.run()




import tables  # pytables for HDF5 support
import os
import sys
import random

from neuroml.hdf5.NetworkContainer import *
from neuromllite import NetworkReader
from neuromllite.utils import print_v, load_json
from pyneuroml.lems import generate_lems_file_for_neuroml

import pprint

pp = pprint.PrettyPrinter(depth=6)

DUMMY_CELL = "dummy_cell"
DEFAULT_NEST_SPIKE_SYN = "DEFAULT_NEST_SPIKE_SYN"

REPORT_FILE = "report.txt"


def _get_default_nest_syn(nml_doc):

    if nml_doc.get_by_id(DEFAULT_NEST_SPIKE_SYN):
        return nml_doc.get_by_id(DEFAULT_NEST_SPIKE_SYN)

    from neuroml import AlphaCurrSynapse

    nest_syn = AlphaCurrSynapse(id=DEFAULT_NEST_SPIKE_SYN, tau_syn="2")
    nml_doc.alpha_curr_synapses.append(nest_syn)
    return nest_syn


def _parse_entry(w):
    """
    Check whether it's an int, float or string & return with that type
    """
    try:
        return int(w)
    except:
        try:
            return float(w)
        except:
            return w


def load_csv_props(info_file):
    """
    Load a generic csv file as used in Sonata
    """
    info = {}
    columns = {}
    for line in open(info_file):
        w = line.split()
        if len(columns) == 0:
            for i in range(len(w)):
                columns[i] = _parse_entry(w[i])
        else:
            info[int(w[0])] = {}
            for i in range(len(w)):
                if i != 0:
                    info[int(w[0])][columns[i]] = _parse_entry(w[i])
    return info


def _matches_node_set_props(type_info, node_set_props):
    """
    Check whether the node_set properties match the given model type definition
    """
    matches = None
    for key in node_set_props:
        ns_val = node_set_props[key]
        if key in type_info:
            if ns_val == type_info[key]:
                if matches:
                    matches = matches and True
                else:
                    matches = True
            else:
                matches = False
    return matches


class SonataReader(NetworkReader):
    """
    Main class for reading a Sonata model. For typical usage, see main method below
    """

    component_objects = {}  # Store cell ids vs objects, e.g. NeuroML2 based object

    nml_includes = ["PyNN.xml"]

    def __init__(self, **parameters):

        print_v("Creating SonataReader with %s..." % parameters)
        self.parameters = parameters

        self.current_sonata_pop = None
        self.current_node_group = None
        self.cell_info = {}
        self.node_set_mappings = {}

        self.pop_comp_info = {}
        self.syn_comp_info = {}
        self.input_comp_info = {}
        self.nml_pop_vs_comps = {}
        self.nml_pops_having_locations = []
        self.nml_ids_vs_gids = {}

        self.init_substitutes = {}
        self.substitutes = {}

        self.current_edge = None

        self.pre_pop = None
        self.post_pop = None

        self.myrandom = random.Random(12345)

    def subs(self, path):
        """
        Search the strings in a config file for a substitutable value, e.g.
        "morphologies_dir": "$COMPONENT_DIR/morphologies",
        """
        # print_v('Checking for: \n  %s, \n  %s \n  in %s'%(self.substitutes,self.init_substitutes,path))
        if type(path) == int or type(path) == float:
            return path
        for s in self.init_substitutes:
            if path.startswith(s):
                path = path.replace(s, self.init_substitutes[s], 1)
        # print_v('  So far: %s'%path)
        for s in self.substitutes:
            if s in path:
                path = path.replace(s, self.substitutes[s])
        # print_v('  Returning: %s'%path)
        return path

    def parse(self, handler):
        """
        Main method to parse the Sonata files and call the appropriate methods
        in the handler
        """

        ########################################################################
        #  load the main configuration scripts

        main_config_filename = os.path.abspath(self.parameters["filename"])

        config = load_json(main_config_filename)

        self.init_substitutes = {
            ".": "%s/" % os.path.dirname(main_config_filename),
            "../": "%s/" % os.path.dirname(os.path.dirname(main_config_filename)),
        }

        self.substitutes = {
            "${configdir}": "%s" % os.path.dirname(main_config_filename)
        }

        if "network" in config:
            self.network_config = load_json(self.subs(config["network"]))
        else:
            self.network_config = config

        if "simulation" in config:
            self.simulation_config = load_json(self.subs(config["simulation"]))
        else:
            self.simulation_config = None

        for m in self.network_config["manifest"]:
            path = self.subs(self.network_config["manifest"][m])
            self.substitutes[m] = path

        if "id" in self.parameters:
            id = self.parameters["id"]
        else:
            id = "SonataNetwork"

        if id[0].isdigit():  # id like 9_cells is not a valid id for NeuroML
            id = "NML2_%s" % id

        ########################################################################
        #  Feed the handler the info on the network

        self.handler = handler

        notes = "Network read in from Sonata: %s" % main_config_filename
        handler.handle_document_start(id, notes)

        handler.handle_network(id, notes)

        self.node_types = {}

        ########################################################################
        #  Get info from nodes files

        for n in self.network_config["networks"]["nodes"]:
            nodes_file = self.subs(n["nodes_file"])
            node_types_file = self.subs(n["node_types_file"])

            print_v("\nLoading nodes from %s and %s" % (nodes_file, node_types_file))

            h5file = tables.open_file(nodes_file, mode="r")

            print_v("Opened HDF5 file: %s" % (h5file.filename))
            self.parse_group(h5file.root.nodes)
            h5file.close()
            self.node_types[self.current_sonata_pop] = load_csv_props(node_types_file)
            self.current_sonata_pop = None

        ########################################################################
        #  Get info from edges files

        self.edges_info = {}
        self.conn_info = {}

        if "edges" in self.network_config["networks"]:
            for e in self.network_config["networks"]["edges"]:
                edges_file = self.subs(e["edges_file"])
                edge_types_file = self.subs(e["edge_types_file"])

                print_v(
                    "\nLoading edges from %s and %s" % (edges_file, edge_types_file)
                )

                h5file = tables.open_file(edges_file, mode="r")

                print_v("Opened HDF5 file: %s" % (h5file.filename))
                self.parse_group(h5file.root.edges)
                h5file.close()
                self.edges_info[self.current_edge] = load_csv_props(edge_types_file)
                self.current_edge = None

        ########################################################################
        #  Use extracted node/cell info to create populations

        for sonata_pop in self.cell_info:
            types_vs_pops = {}

            for type in self.cell_info[sonata_pop]["type_count"]:
                node_type_info = self.node_types[sonata_pop][type]

                model_name_type = (
                    node_type_info["model_name"]
                    if "model_name" in node_type_info
                    else (
                        node_type_info["pop_name"]
                        if "pop_name" in node_type_info
                        else node_type_info["model_type"]
                    )
                )

                model_type = node_type_info["model_type"]
                model_template = (
                    node_type_info["model_template"]
                    if "model_template" in node_type_info
                    else "- None -"
                )

                nml_pop_id = "%s_%s_%s" % (sonata_pop, model_name_type, type)

                print_v(
                    " - Adding population: %s which has model info: %s"
                    % (nml_pop_id, node_type_info)
                )

                size = self.cell_info[sonata_pop]["type_count"][type]

                if model_type == "point_process" and model_template == "nrn:IntFire1":
                    raise Exception(
                        "Point process model not currently supported: %s\nTry expressing the I&F cell in NEST format with nest:iaf_psc_alpha"
                        % model_template
                    )
                    pop_comp = "cell_%s" % nml_pop_id  # model_template.replace(':','_')
                    self.pop_comp_info[pop_comp] = {}
                    self.pop_comp_info[pop_comp]["model_type"] = model_type

                    dynamics_params_file = (
                        self.subs(
                            self.network_config["components"]["point_neuron_models_dir"]
                        )
                        + "/"
                        + node_type_info["dynamics_params"]
                    )
                    self.pop_comp_info[pop_comp]["dynamics_params"] = load_json(
                        dynamics_params_file
                    )

                elif (
                    model_type == "point_process"
                    and model_template == "nest:iaf_psc_alpha"
                ):
                    pop_comp = (
                        "cell_%s" % nml_pop_id
                    )  # = model_template.replace(':','_')
                    self.pop_comp_info[pop_comp] = {}
                    self.pop_comp_info[pop_comp]["model_type"] = model_type
                    self.pop_comp_info[pop_comp]["model_template"] = model_template

                    dynamics_params_file = (
                        self.subs(
                            self.network_config["components"]["point_neuron_models_dir"]
                        )
                        + "/"
                        + node_type_info["dynamics_params"]
                    )
                    self.pop_comp_info[pop_comp]["dynamics_params"] = load_json(
                        dynamics_params_file
                    )
                else:
                    pop_comp = DUMMY_CELL
                    self.pop_comp_info[pop_comp] = {}
                    self.pop_comp_info[pop_comp]["model_type"] = pop_comp

                self.nml_pop_vs_comps[nml_pop_id] = pop_comp

                properties = {}

                properties["type_id"] = type
                properties["sonata_population"] = sonata_pop
                properties["region"] = sonata_pop

                for i in node_type_info:
                    properties[i] = node_type_info[i]
                    if i == "ei":
                        properties["type"] = node_type_info[i].upper()

                color = "%s %s %s" % (
                    self.myrandom.random(),
                    self.myrandom.random(),
                    self.myrandom.random(),
                )

                try:
                    import opencortex.utils.color as occ

                    interneuron = "SOM" in nml_pop_id or "PV" in nml_pop_id
                    if "L23" in nml_pop_id:
                        color = (
                            occ.L23_INTERNEURON
                            if interneuron
                            else occ.L23_PRINCIPAL_CELL
                        )
                        pop.properties.append(neuroml.Property("region", "L23"))
                    if "L4" in nml_pop_id:
                        color = (
                            occ.L4_INTERNEURON if interneuron else occ.L4_PRINCIPAL_CELL
                        )
                        pop.properties.append(neuroml.Property("region", "L4"))
                    if "L5" in nml_pop_id:
                        color = (
                            occ.L5_INTERNEURON if interneuron else occ.L5_PRINCIPAL_CELL
                        )
                        pop.properties.append(neuroml.Property("region", "L5"))
                    if "L6" in nml_pop_id:
                        color = (
                            occ.L6_INTERNEURON if interneuron else occ.L6_PRINCIPAL_CELL
                        )
                        pop.properties.append(neuroml.Property("region", "L6"))
                except:
                    pass  # Don't specify a particular color, use random, not a problem...

                properties["color"] = color
                if True or not "locations" in self.cell_info[sonata_pop]["0"]:

                    properties = {}  #############  temp for LEMS...

                if model_type != "virtual":

                    self.handler.handle_population(
                        nml_pop_id,
                        pop_comp,
                        size,
                        component_obj=None,
                        properties=properties,
                    )

                types_vs_pops[type] = nml_pop_id
            self.cell_info[sonata_pop]["pop_count"] = {}
            self.cell_info[sonata_pop]["pop_map"] = {}

            for i in self.cell_info[sonata_pop]["types"]:

                pop = types_vs_pops[self.cell_info[sonata_pop]["types"][i]]

                if not pop in self.cell_info[sonata_pop]["pop_count"]:
                    self.cell_info[sonata_pop]["pop_count"][pop] = 0

                index = self.cell_info[sonata_pop]["pop_count"][pop]
                self.cell_info[sonata_pop]["pop_map"][i] = (pop, index)

                if not pop in self.nml_ids_vs_gids:
                    self.nml_ids_vs_gids[pop] = {}
                self.nml_ids_vs_gids[pop][index] = (sonata_pop, i)

                if i in self.cell_info[sonata_pop]["0"]["locations"]:
                    if not pop in self.nml_pops_having_locations:
                        self.nml_pops_having_locations.append(pop)
                    pos = self.cell_info[sonata_pop]["0"]["locations"][i]
                    # print('Adding pos %i: %s'%(i,pos))
                    self.handler.handle_location(
                        index,
                        pop,
                        pop_comp,
                        pos["x"] if "x" in pos and pos["x"] is not None else 0,
                        pos["y"] if "y" in pos and pos["y"] is not None else 0,
                        pos["z"] if "z" in pos and pos["z"] is not None else 0,
                    )

                self.cell_info[sonata_pop]["pop_count"][pop] += 1

        ########################################################################
        #  Load simulation info into self.simulation_config

        if self.simulation_config:

            if self.simulation_config:
                for m in self.simulation_config["manifest"]:
                    path = self.subs(self.simulation_config["manifest"][m])
                    self.substitutes[m] = path

            for s1 in ["output"]:
                for k in self.simulation_config[s1]:
                    self.simulation_config[s1][k] = self.subs(
                        self.simulation_config[s1][k]
                    )

            for s1 in ["inputs"]:
                for s2 in self.simulation_config[s1]:
                    for k in self.simulation_config[s1][s2]:
                        self.simulation_config[s1][s2][k] = self.subs(
                            self.simulation_config[s1][s2][k]
                        )

            if "node_sets_file" in self.simulation_config:
                node_sets = load_json(
                    self.subs(self.simulation_config["node_sets_file"])
                )
                self.simulation_config["node_sets"] = node_sets

            if not "node_sets" in self.simulation_config:
                self.simulation_config["node_sets"] = {}

            for sonata_pop in self.cell_info:
                self.node_set_mappings[sonata_pop] = {}
                for sindex in self.cell_info[sonata_pop]["pop_map"]:
                    nml_pop = self.cell_info[sonata_pop]["pop_map"][sindex][0]
                    nml_index = self.cell_info[sonata_pop]["pop_map"][sindex][1]

                    # Add all in this sonata_pop to a 'node_set' named after the sonata_pop
                    if not nml_pop in self.node_set_mappings[sonata_pop]:
                        self.node_set_mappings[sonata_pop][nml_pop] = []
                    self.node_set_mappings[sonata_pop][nml_pop].append(nml_index)

            # pp.pprint(self.simulation_config)
            # pp.pprint(self.pop_comp_info)

            for node_set in self.simulation_config["node_sets"]:
                self.node_set_mappings[node_set] = {}
                node_set_props = self.simulation_config["node_sets"][node_set]
                # print_v('===========Checking which cells in pops match node_set: %s = %s'%(node_set,node_set_props))

                for sonata_pop in self.cell_info:
                    for sindex in self.cell_info[sonata_pop]["pop_map"]:
                        # print('Does %s %s match %s?'%(sonata_pop, sindex, node_set_props))

                        type = self.cell_info[sonata_pop]["types"][sindex]
                        type_info = self.node_types[sonata_pop][type]
                        nml_pop = self.cell_info[sonata_pop]["pop_map"][sindex][0]
                        nml_index = self.cell_info[sonata_pop]["pop_map"][sindex][1]

                        if (
                            "population" in node_set_props
                            and node_set_props["population"] == sonata_pop
                        ):
                            if (
                                "node_id" in node_set_props
                                and sindex in node_set_props["node_id"]
                            ):
                                if not nml_pop in self.node_set_mappings[node_set]:
                                    self.node_set_mappings[node_set][nml_pop] = []
                                self.node_set_mappings[node_set][nml_pop].append(
                                    nml_index
                                )

                        matches = _matches_node_set_props(type_info, node_set_props)
                        # print_v('Node %i in %s (NML: %s[%i]) has type %s (%s); matches: %s'%(sindex, sonata_pop, nml_pop, nml_index, type, type_info, matches))
                        if matches:
                            if not nml_pop in self.node_set_mappings[node_set]:
                                self.node_set_mappings[node_set][nml_pop] = []
                            self.node_set_mappings[node_set][nml_pop].append(nml_index)

            ##pp.pprint(self.node_set_mappings)

        ########################################################################
        #  Extract info from inputs in simulation_config
        # pp.pprint(self.simulation_config)

        for input in self.simulation_config["inputs"]:
            info = self.simulation_config["inputs"][input]
            # print_v(" - Adding input: %s which has info: %s"%(input, info))

            self.input_comp_info[input] = {}
            self.input_comp_info[input][info["input_type"]] = {}

            node_set = info["node_set"]

            if info["input_type"] == "current_clamp":

                comp = "PG_%s" % input
                self.input_comp_info[input][info["input_type"]][comp] = {
                    "amp": info["amp"],
                    "delay": info["delay"],
                    "duration": info["duration"],
                }

                for nml_pop_id in self.node_set_mappings[node_set]:

                    input_list_id = "il_%s_%s" % (input, nml_pop_id)
                    indices = self.node_set_mappings[node_set][nml_pop_id]

                    self.handler.handle_input_list(
                        input_list_id, nml_pop_id, comp, len(indices)
                    )
                    count = 0
                    for index in indices:
                        self.handler.handle_single_input(
                            input_list_id, count, cellId=index, segId=0, fract=0.5
                        )
                        count += 1

            elif info["input_type"] == "spikes":
                node_info = self.cell_info[node_set]

                from pyneuroml.plot.PlotSpikes import read_sonata_spikes_hdf5_file
                from pyneuroml.plot.PlotSpikes import POP_NAME_SPIKEFILE_WITH_GIDS

                ids_times = read_sonata_spikes_hdf5_file(self.subs(info["input_file"]))
                for id in ids_times[POP_NAME_SPIKEFILE_WITH_GIDS]:
                    times = ids_times[POP_NAME_SPIKEFILE_WITH_GIDS][id]
                    if id in node_info["pop_map"]:
                        nml_pop_id, cell_id = node_info["pop_map"][id]
                        print_v(
                            "Cell %i in Sonata node set %s (cell %s in nml pop %s) has %i spikes"
                            % (id, node_set, nml_pop_id, cell_id, len(times))
                        )

                        component = "%s__%i" % (nml_pop_id, cell_id)

                        self.input_comp_info[input][info["input_type"]][component] = {
                            "id": cell_id,
                            "times": times,
                        }

                        """
                        input_list_id = 'il_%s_%i'%(input,cell_id)
                        self.handler.handle_input_list(input_list_id,
                                                       nml_pop_id,
                                                       component,
                                                       1)

                        self.handler.handle_single_input(input_list_id,
                                                          0,
                                                          cellId = cell_id,
                                                          segId = 0,
                                                          fract = 0.5)
                                                         """
                    else:
                        print_v(
                            "Cell %i in Sonata node set %s NOT FOUND!" % (id, node_set)
                        )

            else:
                raise Exception(
                    "Sonata input type not yet supported: %s" % (info["input_type"])
                )

        ########################################################################
        #  Use extracted edge info to create connections

        projections_created = []
        for conn in self.conn_info:

            pre_node = self.conn_info[conn]["pre_node"]
            post_node = self.conn_info[conn]["post_node"]

            for i in range(len(self.conn_info[conn]["pre_id"])):
                pre_id = self.conn_info[conn]["pre_id"][i]
                post_id = self.conn_info[conn]["post_id"][i]
                nsyns = (
                    self.conn_info[conn]["nsyns"][i]
                    if "nsyns" in self.conn_info[conn]
                    else 1
                )
                type = self.conn_info[conn]["edge_type_id"][i]
                # print_v('   Conn with %i syns, type %s: %s(%s) -> %s(%s)'%(nsyns,type,pre_node,pre_id,post_node,post_id))
                pre_pop, pre_i = self.cell_info[pre_node]["pop_map"][pre_id]
                post_pop, post_i = self.cell_info[post_node]["pop_map"][post_id]
                # print_v('   Mapped: Conn %s(%s) -> %s(%s)'%(pre_pop,pre_i,post_pop,post_i))
                # print self.edges_info[conn][type]
                # print self.cell_info[pre_node]
                # print 11
                # print self.node_types[pre_node]
                # print 22
                cell_type_pre = self.cell_info[pre_node]["types"][pre_id]
                # print cell_type_pre
                # print 444
                pop_type_pre = self.node_types[pre_node][cell_type_pre]["model_type"]
                # print pop_type_pre
                # print 333

                synapse = self.edges_info[conn][type]["dynamics_params"].split(".")[0]
                self.syn_comp_info[synapse] = {}
                # print self.edges_info[conn][type]
                # pp.pprint(self.init_substitutes)
                # pp.pprint(self.substitutes)
                dynamics_params_file = (
                    self.subs(self.network_config["components"]["synaptic_models_dir"])
                    + "/"
                    + self.edges_info[conn][type]["dynamics_params"]
                )
                # print_v('Adding syn %s (at %s)'%(self.edges_info[conn][type]['dynamics_params'], dynamics_params_file))

                # TODO: don't load this file every connection!!!
                self.syn_comp_info[synapse]["dynamics_params"] = load_json(
                    dynamics_params_file
                )
                proj_id = "%s_%s_%s" % (pre_pop, post_pop, synapse)

                sign = (
                    self.syn_comp_info[synapse]["dynamics_params"]["sign"]
                    if "sign" in self.syn_comp_info[synapse]["dynamics_params"]
                    else 1
                )
                weight = (
                    self.edges_info[conn][type]["syn_weight"]
                    if "syn_weight" in self.edges_info[conn][type]
                    else 1.0
                )

                syn_weight_edge_group_0 = (
                    self.conn_info[conn]["syn_weight_edge_group_0"][i]
                    if "syn_weight_edge_group_0" in self.conn_info[conn]
                    else None
                )

                # Assume this overrides value from csv file...
                if syn_weight_edge_group_0:
                    weight = syn_weight_edge_group_0

                # print_v('Adding syn %s (at %s), weight: %s, sign: %s, nsyns: %s'%(self.edges_info[conn][type]['dynamics_params'], dynamics_params_file, weight, sign, nsyns))

                weight_scale = 0.001
                if "level_of_detail" in self.syn_comp_info[synapse]["dynamics_params"]:
                    weight_scale = 1

                weight = weight_scale * sign * weight * nsyns

                delay = (
                    self.edges_info[conn][type]["delay"]
                    if "delay" in self.edges_info[conn][type]
                    else 0
                )

                if not pop_type_pre == "virtual":
                    if not proj_id in projections_created:

                        self.handler.handle_projection(
                            proj_id, pre_pop, post_pop, synapse
                        )

                        projections_created.append(proj_id)

                    self.handler.handle_connection(
                        proj_id,
                        i,
                        pre_pop,
                        post_pop,
                        synapse,
                        pre_i,
                        post_i,
                        weight=weight,
                        delay=delay,
                    )

                else:
                    component = "%s__%i" % (pre_pop, pre_i)
                    # print_v('   ---  Connecting %s to %s[%s]'%(component, post_pop, post_i))

                    # self.input_comp_info[input][info['input_type']][component] ={'id': cell_id, 'times': times}

                    input_list_id = "il_%s_%s_%i_%i" % (component, post_pop, post_i, i)

                    self.handler.handle_input_list(
                        input_list_id, post_pop, component, 1
                    )

                    self.handler.handle_single_input(
                        input_list_id,
                        0,
                        cellId=post_i,
                        segId=0,
                        fract=0.5,
                        weight=weight,
                    )

        """
        print('~~~~~~~~~~~~~~~')
        print('node_types:')
        pp.pprint(self.node_types)
        print('~~~~~~~~~~~~~~~')
        print('cell_info:')
        pp.pprint(self.cell_info)
        print('================')"""

    def parse_group(self, g):
        # print("+++++++++++++++Parsing group: "+ str(g)+", name: "+g._v_name)

        for node in g:
            # print("   ------Sub node: %s, class: %s, name: %s (parent: %s)"   % (node,node._c_classid,node._v_name, g._v_name))

            if node._c_classid == "GROUP":
                if g._v_name == "nodes":
                    son_pop_id = node._v_name.replace("-", "_")
                    self.current_sonata_pop = son_pop_id
                    self.cell_info[self.current_sonata_pop] = {}
                    self.cell_info[self.current_sonata_pop]["types"] = {}
                    self.cell_info[self.current_sonata_pop]["type_count"] = {}

                if g._v_name == self.current_sonata_pop:
                    node_group = node._v_name
                    self.current_node_group = node_group
                    self.cell_info[self.current_sonata_pop][
                        self.current_node_group
                    ] = {}
                    self.cell_info[self.current_sonata_pop][self.current_node_group][
                        "locations"
                    ] = {}

                if g._v_name == "edges":
                    edge_id = node._v_name.replace("-", "_")
                    # print('  Found edge: %s'%edge_id)
                    self.current_edge = edge_id
                    self.conn_info[self.current_edge] = {}

                if g._v_name == self.current_edge:

                    self.current_pre_node = g._v_name.split("_to_")[0]
                    self.current_post_node = g._v_name.split("_to_")[1]
                    # print('  Found edge %s -> %s'%(self.current_pre_node, self.current_post_node))
                    self.conn_info[self.current_edge][
                        "pre_node"
                    ] = self.current_pre_node
                    self.conn_info[self.current_edge][
                        "post_node"
                    ] = self.current_post_node

                self.parse_group(node)

            if self._is_dataset(node):
                self.parse_dataset(node)

        self.current_node_group = None

    def _is_dataset(self, node):
        return node._c_classid == "ARRAY" or node._c_classid == "CARRAY"

    def parse_dataset(self, d):
        # print_v("Parsing dataset/array: %s; at node: %s, node_group %s"%(str(d), self.current_sonata_pop, self.current_node_group))

        if self.current_node_group:  # e.g. parent group is 0 with child datasets x,y,z
            for i in range(0, d.shape[0]):
                if (
                    not i
                    in self.cell_info[self.current_sonata_pop][self.current_node_group][
                        "locations"
                    ]
                ):
                    self.cell_info[self.current_sonata_pop][self.current_node_group][
                        "locations"
                    ][i] = {}
                self.cell_info[self.current_sonata_pop][self.current_node_group][
                    "locations"
                ][i][d.name] = d[i]

        elif (
            self.current_sonata_pop
        ):  # e.g. parent group is cortex with child datasets node_id etc.

            if d.name == "node_group_id":
                for i in range(0, d.shape[0]):
                    if not d[i] == 0:
                        raise Exception(
                            "Error: currently only support node_group_id==0!"
                        )
            if d.name == "node_id":
                for i in range(0, d.shape[0]):
                    if not d[i] == i:
                        raise Exception(
                            "Error: currently only support dataset node_id when index is same as node_id (fails in %s)...!"
                            % d
                        )
            if d.name == "node_type_id":
                for i in range(0, d.shape[0]):
                    self.cell_info[self.current_sonata_pop]["types"][i] = d[i]
                    if (
                        not d[i]
                        in self.cell_info[self.current_sonata_pop]["type_count"]
                    ):
                        self.cell_info[self.current_sonata_pop]["type_count"][d[i]] = 0
                    self.cell_info[self.current_sonata_pop]["type_count"][d[i]] += 1

        elif d.name == "source_node_id":
            self.conn_info[self.current_edge]["pre_id"] = [i for i in d]
        elif d.name == "edge_type_id":
            self.conn_info[self.current_edge]["edge_type_id"] = [int(i) for i in d]
        elif d.name == "target_node_id":
            self.conn_info[self.current_edge]["post_id"] = [i for i in d]
        elif d.name == "nsyns":
            self.conn_info[self.current_edge]["nsyns"] = [i for i in d]
        elif d.name == "edge_group_id":
            for i in range(0, d.shape[0]):
                if not d[i] == 0:
                    raise Exception("Error: currently only support edge_group_id==0!")
        elif d.name == "syn_weight":
            # Has to be edge_group_id==0, as above check would fail...
            self.conn_info[self.current_edge]["syn_weight_edge_group_0"] = [
                i for i in d
            ]
        else:
            print_v("Unhandled dataset: %s" % d.name)

    def add_neuroml_components(self, nml_doc):
        """
        Based on cell & synapse properties found, create the corresponding NeuroML components
        """

        is_nest = False
        print_v("Adding NeuroML cells to: %s" % nml_doc.id)
        # pp.pprint(self.pop_comp_info)

        for c in self.pop_comp_info:
            info = self.pop_comp_info[c]

            model_template = (
                info["model_template"]
                if "model_template" in info
                else (
                    info["dynamics_params"]["type"]
                    if "dynamics_params" in info
                    else info["model_type"]
                )
            )

            print_v(" - Adding %s: %s" % (model_template, info))

            if (
                info["model_type"] == "point_process"
                and model_template == "nest:iaf_psc_alpha"
            ):

                is_nest = True
                from neuroml import IF_curr_alpha

                pynn0 = IF_curr_alpha(
                    id=c,
                    cm=info["dynamics_params"]["C_m"] / 1000.0,
                    i_offset="0",
                    tau_m=info["dynamics_params"]["tau_m"],
                    tau_refrac=info["dynamics_params"]["t_ref"],
                    tau_syn_E="1",
                    tau_syn_I="1",
                    v_init="-70",
                    v_reset=info["dynamics_params"]["V_reset"],
                    v_rest=info["dynamics_params"]["E_L"],
                    v_thresh=info["dynamics_params"]["V_th"],
                )

                nml_doc.IF_curr_alpha.append(pynn0)

            elif (
                info["model_type"] == "point_process"
                and model_template == "NEURON_IntFire1"
            ):

                contents = """<Lems>
    <intFire1Cell id="%s" thresh="1mV" reset="0mV" tau="%sms" refract="%sms"/>
</Lems>""" % (
                    c,
                    info["dynamics_params"]["tau"] * 1000,
                    info["dynamics_params"]["refrac"] * 1000,
                )

                cell_file_name = "%s.xml" % c
                cell_file = open(cell_file_name, "w")
                cell_file.write(contents)
                cell_file.close()

                self.nml_includes.append(cell_file_name)
                self.nml_includes.append(
                    "../../../examples/sonatatest/IntFireCells.xml"
                )

            else:

                from neuroml import IafRefCell

                IafRefCell0 = IafRefCell(
                    id=DUMMY_CELL,
                    C=".2 nF",
                    thresh="1mV",
                    reset="0mV",
                    refract="3ms",
                    leak_conductance="1.2 nS",
                    leak_reversal="0mV",
                )

                print_v("   - Adding: %s" % IafRefCell0)
                nml_doc.iaf_ref_cells.append(IafRefCell0)

        print_v("Adding NeuroML synapses to: %s" % nml_doc.id)

        # pp.pprint(self.syn_comp_info)

        for s in self.syn_comp_info:
            dyn_params = self.syn_comp_info[s]["dynamics_params"]

            print_v("     -  Syn: %s: %s" % (s, dyn_params))

            if (
                "level_of_detail" in dyn_params
                and dyn_params["level_of_detail"] == "exp2syn"
            ):
                from neuroml import ExpTwoSynapse

                syn = ExpTwoSynapse(
                    id=s,
                    gbase="1nS",
                    erev="%smV" % dyn_params["erev"],
                    tau_rise="%sms" % dyn_params["tau1"],
                    tau_decay="%sms" % dyn_params["tau2"],
                )

                # print("Adding syn: %s"%syn)
                nml_doc.exp_two_synapses.append(syn)

            elif (
                "level_of_detail" in dyn_params
                and dyn_params["level_of_detail"] == "instanteneous"
            ):

                contents = """<Lems>
    <impulseSynapse id="%s"/>
</Lems>""" % (
                    s
                )

                syn_file_name = "%s.xml" % s
                syn_file = open(syn_file_name, "w")
                syn_file.write(contents)
                syn_file.close()

                self.nml_includes.append(syn_file_name)
                # self.nml_includes.append('../examples/sonatatest/IntFireCells.xml')

            else:
                from neuroml import AlphaCurrSynapse

                pynnSynn0 = AlphaCurrSynapse(id=s, tau_syn="2")
                # print("Adding syn: %s"%pynnSynn0)
                nml_doc.alpha_curr_synapses.append(pynnSynn0)

        print_v("Adding NeuroML inputs to: %s" % nml_doc.id)

        # pp.pprint(self.input_comp_info)

        for input in self.input_comp_info:
            for input_type in self.input_comp_info[input]:
                if input_type == "spikes":
                    for comp_id in self.input_comp_info[input][input_type]:
                        info = self.input_comp_info[input][input_type][comp_id]
                        print_v("Adding input %s: %s" % (comp_id, info.keys()))

                        nest_syn = _get_default_nest_syn(nml_doc)
                        from neuroml import TimedSynapticInput, Spike

                        tsi = TimedSynapticInput(
                            id=comp_id,
                            synapse=nest_syn.id,
                            spike_target="./%s" % nest_syn.id,
                        )
                        nml_doc.timed_synaptic_inputs.append(tsi)
                        for ti in range(len(info["times"])):
                            tsi.spikes.append(
                                Spike(id=ti, time="%sms" % info["times"][ti])
                            )
                elif input_type == "current_clamp":
                    from neuroml import PulseGenerator

                    for comp_id in self.input_comp_info[input][input_type]:
                        info = self.input_comp_info[input][input_type][comp_id]
                        # TODO remove when https://github.com/AllenInstitute/sonata/issues/42 is fixed!
                        amp_template = "%spA" if is_nest else "%snA"  #
                        pg = PulseGenerator(
                            id=comp_id,
                            delay="%sms" % info["delay"],
                            duration="%sms" % info["duration"],
                            amplitude=amp_template % info["amp"],
                        )
                        nml_doc.pulse_generators.append(pg)

    def generate_lems_file(self, nml_file_name, nml_doc):
        """
        Generate a LEMS file to use in simulations of the NeuroML file
        """

        # pp.pprint(self.simulation_config)
        # pp.pprint(self.pop_comp_info)
        # pp.pprint(self.node_set_mappings)

        if "output" in self.simulation_config:
            gen_spike_saves_for_all_somas = True

        target = nml_doc.networks[0].id
        sim_id = "Sim_%s" % target

        duration = self.simulation_config["run"]["tstop"]
        dt = self.simulation_config["run"]["dt"]
        lems_file_name = "LEMS_%s.xml" % sim_id
        target_dir = "./"
        gen_saves_for_quantities = {}
        gen_plots_for_quantities = {}

        if "reports" in self.simulation_config:
            if "membrane_potential" in self.simulation_config["reports"]:
                mp = self.simulation_config["reports"]["membrane_potential"]
                node_set = self.node_set_mappings[mp["cells"]]
                for nml_pop in node_set:
                    comp = self.nml_pop_vs_comps[nml_pop]
                    ids = node_set[nml_pop]
                    display = "Voltages_%s" % nml_pop
                    file_name = "%s.v.dat" % nml_pop

                    for id in ids:
                        quantity = "%s/%i/%s/%s" % (nml_pop, id, comp, "v")
                        if not nml_pop in self.nml_pops_having_locations:
                            quantity = "%s[%i]/%s" % (nml_pop, id, "v")

                        if not display in gen_plots_for_quantities:
                            gen_plots_for_quantities[display] = []
                        gen_plots_for_quantities[display].append(quantity)
                        if not file_name in gen_saves_for_quantities:
                            gen_saves_for_quantities[file_name] = []
                        gen_saves_for_quantities[file_name].append(quantity)

        generate_lems_file_for_neuroml(
            sim_id,
            nml_file_name,
            target,
            duration,
            dt,
            lems_file_name,
            target_dir,
            include_extra_files=self.nml_includes,
            gen_plots_for_all_v=False,
            plot_all_segments=False,
            gen_plots_for_quantities=gen_plots_for_quantities,  #  Dict with displays vs lists of quantity paths
            gen_saves_for_all_v=False,
            save_all_segments=False,
            gen_saves_for_quantities=gen_saves_for_quantities,  #  List of populations, all pops if = []
            gen_spike_saves_for_all_somas=gen_spike_saves_for_all_somas,
            report_file_name=REPORT_FILE,
            copy_neuroml=True,
            verbose=True,
        )

        return lems_file_name


def get_neuroml_from_sonata(sonata_filename, id, generate_lems=True, format="xml"):
    """
    Return a NeuroMLDocument with (most of) the contents of the Sonata model
    """

    from neuroml.hdf5.NetworkBuilder import NetworkBuilder

    neuroml_handler = NetworkBuilder()

    sr = SonataReader(filename=sonata_filename, id=id)

    sr.parse(neuroml_handler)

    nml_doc = neuroml_handler.get_nml_doc()

    sr.add_neuroml_components(nml_doc)

    if format == "xml":
        nml_file_name = "%s.net.nml" % id
        from neuroml.writers import NeuroMLWriter

        NeuroMLWriter.write(nml_doc, nml_file_name)

    elif format == "hdf5":
        nml_file_name = "%s.net.nml.h5" % id
        from neuroml.writers import NeuroMLHdf5Writer

        NeuroMLHdf5Writer.write(nml_doc, nml_file_name)

    print_v("Written to: %s" % nml_file_name)

    if generate_lems:
        lems_file_name = sr.generate_lems_file(nml_file_name, nml_doc)

        return sr, lems_file_name, nml_file_name, nml_doc

    return nml_doc


def process_args():
    """
    Parse command-line arguments.
    """
    import argparse

    parser = argparse.ArgumentParser(
        description="Read Sonata format configuration file and execute using jNeuroML export formats"
    )

    parser.add_argument(
        "network_reference",
        type=str,
        metavar="<network reference>",
        help="Reference/id for network (to be used for NeuroML files)",
    )

    parser.add_argument(
        "sonata_config_file",
        type=str,
        metavar="<sonata config file>",
        help="Sonata configuration file",
    )

    parser.add_argument(
        "-h5",
        action="store_true",
        default=False,
        help="Save NeuroML as HDF5, as opposed to XML",
    )

    parser.add_argument(
        "-jnml",
        action="store_true",
        default=False,
        help="Execute the generated LEMS/NeuroML2 model with jNeuroML",
    )

    parser.add_argument(
        "-neuron",
        action="store_true",
        default=False,
        help="Execute the generated LEMS/NeuroML2 model with jNeuroML_NEURON",
    )

    return parser.parse_args()


def _get_nml_pop_id(quantity):

    if "[" in quantity:
        nml_pop = quantity.split("[")[0]
        nml_index = int(quantity.split("[")[1].split("]")[0])
        return nml_pop, nml_index

    else:
        nml_pop = quantity.split("/")[0]
        nml_index = int(quantity.split("/")[1])
        return nml_pop, nml_index


def run(args):

    print_v(
        "Reading Sonata file: %s and generating network: %s"
        % (args.sonata_config_file, args.network_reference)
    )

    sr, lems_file_name, nml_file_name, nml_doc = get_neuroml_from_sonata(
        args.sonata_config_file,
        args.network_reference,
        generate_lems=True,
        format=("hdf5" if args.h5 else "xml"),
    )

    # pp.pprint(sr.cell_info)
    # pp.pprint(sr.nml_ids_vs_gids)

    print_v("Generated NeuroML 2 network: %s" % (nml_file_name))
    print_v("Generated LEMS file to run network: %s" % (lems_file_name))

    if args.jnml or args.neuron:

        traces = None
        events = None

        if args.jnml:

            from pyneuroml import pynml

            traces, events = pynml.run_lems_with_jneuroml(
                lems_file_name,
                nogui=True,
                verbose=True,
                load_saved_data=True,
                reload_events=True,
            )
        if args.neuron:

            from pyneuroml import pynml

            traces, events = pynml.run_lems_with_jneuroml_neuron(
                lems_file_name,
                nogui=True,
                verbose=True,
                load_saved_data=True,
                reload_events=True,
            )

        if traces:
            trace_file = (
                sr.simulation_config["output"]["output_dir"] + "/membrane_potential.h5"
            )
            print_v("Resaving data to %s" % trace_file)
            import tables  # pytables for HDF5 support

            h5file = tables.open_file(trace_file, mode="w")
            report_grp = h5file.create_group("/", "report")

            node_info = {}

            t = traces["t"]
            time_start = t[0] * 1000.0
            time_stop = t[-1] * 1000.0
            time_dt = (t[1] - t[0]) * 1000.0
            time = [time_start, time_stop, time_dt]

            nml_q_vs_node_id = {}

            for nml_q in traces.keys():

                if nml_q != "t":
                    nml_pop, nml_index = _get_nml_pop_id(nml_q)
                    (sonata_node, sonata_node_id) = sr.nml_ids_vs_gids[nml_pop][
                        nml_index
                    ]
                    nml_q_vs_node_id[nml_q] = sonata_node_id

            ordered = sorted(nml_q_vs_node_id, key=nml_q_vs_node_id.get)

            for nml_q in ordered:

                if nml_q != "t":
                    v = traces[nml_q]
                    nml_pop, nml_index = _get_nml_pop_id(nml_q)
                    (sonata_node, sonata_node_id) = sr.nml_ids_vs_gids[nml_pop][
                        nml_index
                    ]

                    if not sonata_node in node_info:
                        node_info[sonata_node] = {}
                        node_info[sonata_node]["data"] = []
                        node_info[sonata_node]["gids"] = []

                    node_info[sonata_node]["data"].append([vv * 1000.0 for vv in v])
                    node_info[sonata_node]["gids"].append(sonata_node_id)

            for sonata_node in node_info:

                node_grp = h5file.create_group(report_grp, sonata_node)
                mapping_grp = h5file.create_group(node_grp, "mapping")

                # node_grp = h5file.root
                # mapping_grp = h5file.create_group(report_grp, 'mapping')
                gids = node_info[sonata_node]["gids"]
                h5file.create_array(mapping_grp, "gids", gids)
                h5file.create_array(mapping_grp, "node_ids", gids)
                h5file.create_array(
                    mapping_grp, "index_pointer", [i for i in range(len(gids) + 1)]
                )
                h5file.create_array(mapping_grp, "element_ids", [0 for g in gids])
                h5file.create_array(mapping_grp, "element_pos", [0.5 for g in gids])
                h5file.create_array(mapping_grp, "time", time)

                d = h5file.create_array(
                    node_grp, "data", np.array(node_info[sonata_node]["data"]).T
                )
                d.attrs.units = "mV"
                d.attrs.variable_name = "V_m"

            h5file.close()

        if events:
            event_file = (
                sr.simulation_config["output"]["output_dir"]
                + "/"
                + sr.simulation_config["output"]["spikes_file"]
            )
            print_v("Resaving data to %s" % event_file)
            import tables  # pytables for HDF5 support

            h5file = tables.open_file(event_file, mode="w")
            spike_grp = h5file.create_group("/", "spikes")
            gids = []
            spiketimes = []
            for nml_q in events:
                nml_pop, nml_index = _get_nml_pop_id(nml_q)
                (sonata_node, sonata_node_id) = sr.nml_ids_vs_gids[nml_pop][nml_index]
                for t in events[nml_q]:
                    gids.append(sonata_node_id)
                    spiketimes.append(t * 1000.0)

            h5file.create_array(spike_grp, "gids", gids)
            h5file.create_array(spike_grp, "timestamps", spiketimes)

            h5file.close()

        if sr.simulation_config["output"]["log_file"]:
            log_file = (
                sr.simulation_config["output"]["output_dir"]
                + "/"
                + sr.simulation_config["output"]["log_file"]
            )
            from shutil import copyfile

            copyfile(REPORT_FILE, log_file)


def main(args=None):

    if "-test" in sys.argv:
        id = "9_cells"
        id = "300_cells"
        # id = '5_cells_iclamp'

        id = "300_intfire"
        id = "small_intfire"
        id = "small_iclamp"
        ## https://github.com/pgleeson/sonata/tree/intfire
        filename = "../../git/sonatapg/examples/%s/config.json" % id

        id = "ten_cells_spikes2"

        id = "ten_cells_spikes_nrn"
        id = "one_cell_iclamp_nest"
        id = "ten_cells_iclamp_nest"
        id = "ten_cells_spikes_nest"
        filename = (
            "/home/padraig/git/sonatapg/examples/sim_tests/intfire/%s/input/config.json"
            % id
        )
        # id = '300_pointneurons'
        # filename = '/home/padraig/git/sonatapg/examples/%s/config.json'%id

        nml_doc = get_neuroml_from_sonata(filename, id, generate_lems=True)
        """
        nml_file_name = '%s.net.nml'%id
        nml_file_name += '.h5'

        from neuroml.writers import NeuroMLHdf5Writer
        NeuroMLHdf5Writer.write(nml_doc,nml_file_name)
        print('Written to: %s'%nml_file_name) """
    else:

        if args is None:
            args = process_args()
        run(args=args)


if __name__ == "__main__":
    main()



import sys
from bmtk.analyzer.visualization.spikes import plot_spikes


from analyze_output_utils import plot_data

if __name__ == "__main__":

    plot_data(
        "output/membrane_potential.h5", "mV", "Membrane Potential", show_already=True
    )
    # plot_spikes('../input/network/pre_nodes.h5', '../input/network/pre_node_types.csv', 'output/spikes.h5')
    # plot_spikes('../input/network/post_nodes.h5', '../input/network/post_node_types.csv', 'output/spikes.h5')



import h5py
import numpy as np
import matplotlib.pyplot as plt


def plot_data(reports_file, y_axis, title, show_already=True, max_num_traces=None):
    print("Plotting data on %s (%s) from file: %s" % (title, y_axis, reports_file))
    cellvar_h5 = h5py.File(reports_file, "r")

    try:
        top_level = cellvar_h5["/mapping"]
        pop_prefixes = {"GLOBAL": "/"}
    except:
        pop_prefixes = {}
        for pop in cellvar_h5["/report"].keys():
            pop_prefixes = {pop: "/report/%s/" % pop}

    for pop in pop_prefixes:

        gids = np.array(cellvar_h5["%smapping/gids" % pop_prefixes[pop]])

        soma_locs = np.array(
            cellvar_h5["%smapping/index_pointer" % pop_prefixes[pop]]
        )  # location of soma
        t_start = cellvar_h5["%smapping/time" % pop_prefixes[pop]][0]
        t_stop = cellvar_h5["%smapping/time" % pop_prefixes[pop]][1]
        dt = cellvar_h5["%smapping/time" % pop_prefixes[pop]][2]
        time_steps = np.linspace(t_start, t_stop, 1 + (t_stop - t_start) / float(dt))
        print(
            "Time steps %s -> %s, dt: %s (%s points)"
            % (t_start, t_stop, dt, len(time_steps))
        )

        n_plots = len(gids)
        if max_num_traces:
            n_plots = min(max_num_traces, n_plots)
        data_table = np.array(cellvar_h5["%sdata" % pop_prefixes[pop]])
        f, axarr = plt.subplots(n_plots, 1)
        f.suptitle(title)
        for i, (gid, soma_index) in enumerate(zip(gids, soma_locs)):
            if i < n_plots:
                data = data_table[:, [soma_index]]
                ax = axarr if n_plots == 1 else axarr[i]

                print(
                    "For gid %i (%s) there are %i time steps and %i data points"
                    % (gid, pop, len(time_steps), len(data))
                )
                label = "%s: gid %i" % (pop, gid)
                if len(time_steps) == len(data):
                    ax.plot(time_steps, data, label=label)

                    fn = "output/gid_%s.dat" % gid
                    f = open(fn, "w")
                    print("Writing %i points to %s" % (len(time_steps), fn))
                    for ti in range(len(time_steps)):
                        f.write(
                            "%s\t%s\n" % (time_steps[ti] / 1000, data[ti][0] / 1000)
                        )
                    f.close()
                else:
                    ax.plot(data, label=label)

                ax.legend()
                ax.set_ylabel(y_axis)
                if i < n_plots - 1:
                    ax.set_xticklabels([])
                    ax.set_xlabel("ms")

    if show_already:
        plt.show()
